@@ -20,39 +20,40 @@
   the file called "COPYING".
 
   Contact Information:
-  e1000-eedc Mailing List <e1000-eedc@lists.sourceforge.net>
-  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497
+  open-lldp Mailing List <lldp-devel@open-lldp.org>
 
 *******************************************************************************/
 
 #include <syslog.h>
 #include <string.h>
 #include <stdlib.h>
-#include <sys/types.h>
+#include <errno.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
 #include <linux/sockios.h>
 #include <net/if.h>
-#include "common.h"
+#include <unistd.h>
 #include "eloop.h"
 #include "lldpad.h"
 #include "lldp.h"
+#include "lldp/ports.h"
+#include "lldp/agent.h"
+#include "lldp/l2_packet.h"
 #include "lldp_util.h"
 #include "lldp_mod.h"
 #include "lldp_mand_clif.h"
+#include "lldp_med.h"
 #include "dcb_protocol.h"
 #include "dcb_persist_store.h"
 #include "messages.h"
-#include "libconfig.h"
-#include "drv_cfg.h"
 #include "config.h"
-#include "clif_msgs.h"
-#include "lldp/l2_packet.h"
+#include "lldpad_status.h"
+#include "lldp_mod.h"
+#include "event_iface.h"
 
 config_t lldpad_cfg;
-static int cfg_loaded = false;
 
 /*
  * init_cfg - initialze the global lldpad_cfg via config_init
@@ -63,18 +64,27 @@ static int cfg_loaded = false;
  */
 int init_cfg(void)
 {
-	if (cfg_loaded == false) {
-		config_init(&lldpad_cfg);
-		if (!config_read_file(&lldpad_cfg, cfg_file_name))
-			printf("config file failed to load\n");
-		else
-			cfg_loaded = true;
-	} else {
+	const char *p;
+	int err = 1;
+
+	config_init(&lldpad_cfg);
+
+	if (check_cfg_file()) {
+		err = 0;
+		LLDPAD_INFO("%s: failed to create config file\n", __func__);
+	} else if (!config_read_file(&lldpad_cfg, cfg_file_name)) {
+		err = 0;
+		LLDPAD_INFO("%s: config file failed to load\n", __func__);
+	} else if (config_lookup_string(&lldpad_cfg, "version", &p)) {
+		LLDPAD_INFO("%s: config file version incorrect ", __func__);
+		LLDPAD_INFO("rebuild file with correct version.\n");
+		destroy_cfg();
+		remove(cfg_file_name);
 		if (check_cfg_file() ||
-			(!config_read_file(&lldpad_cfg, cfg_file_name)))
-			cfg_loaded = false;
+		    !config_read_file(&lldpad_cfg, cfg_file_name))
+			err = 0;
 	}
-	return cfg_loaded;
+	return err;
 }
 
 /*
@@ -86,32 +96,23 @@ int init_cfg(void)
  */
 void destroy_cfg(void)
 {
-	if (cfg_loaded == true) {
-		config_destroy(&lldpad_cfg);
-		cfg_loaded = false;
-	}
+	config_destroy(&lldpad_cfg);
 }
 
-void scan_port(void *eloop_data, void *user_ctx)
+void scan_port(UNUSED void *eloop_data, UNUSED void *user_ctx)
 {
 	struct port *port;
 	struct if_nameindex *nameidx, *p;
-	struct lldp_module *np;
-	const struct lldp_mod_ops *ops;
-	int err;
 
-	printf("%s: NLMSG dropped, scan ports.\n", __func__);
+	LLDPAD_INFO("%s: NLMSG dropped, scan ports.\n", __func__);
 
 	nameidx = if_nameindex();
 	if (nameidx == NULL) {
-		TRACE("if_nameindex error try again later\n");
+		LLDPAD_DBG("if_nameindex error try again later\n");
 		goto error_out;
 	}
 	p = nameidx;
 
-	if (!init_cfg())
-		goto err_out;
-
 	/* Walk port list looking for devices that are not in if_nameindex.
 	 * If the device is in the port list but not in the if_nameindex
 	 * list then we missed a RTM_DELLINK event and the device is no
@@ -137,29 +138,52 @@ void scan_port(void *eloop_data, void *user_ctx)
 		del = port;
 		port = port->next;
 		if (!found)
-			remove_adapter(del->ifname);
+			remove_port(del->ifname);
 	}
 
 	/* Walk port list looking for devices that should have been added
 	 * to our port list but have not most likely due to a dropped nlmsg.
 	 * At this point we need to add the device and call ops ifup routines.
-	 * Additional we call set_hw_all() for all devices already in the port
-	 * list this should force the hw settings to get update if needed.
+	 * The port enable state needs to be set to match the real link state
+	 * multiple link events and the port state is no longer reliable.
 	 * This is required because we currently do not know if we missed
-	 * IF_OPER_UP or IF_OPER_DORMANT.  And an extra set_hw_all()
-	 * shouldn't hurt anything.
+	 * IF_OPER_UP, IF_OPER_DOWN or IF_OPER_DORMANT. 
 	 */
 	p = nameidx;
 	while (p->if_index != 0) {
-		if (is_valid_lldp_device(p->if_name) &&
-		    check_link_status(p->if_name) > 0) {
-			oper_add_device(p->if_name);
-			set_hw_all(p->if_name);
+		struct lldp_module *np;
+		const struct lldp_mod_ops *ops;
+		char *ifname = p->if_name;
+		struct lldp_agent *agent;
+
+		if (!is_valid_lldp_device(ifname)) {
+			p++;
+			continue;
+		}
+
+		port = port_find_by_name(p->if_name);
+		if (!port)
+			port = add_port(p->if_name);
+
+		if (port && check_link_status(ifname)) {
+			set_port_oper_delay(ifname);
+			oper_add_device(ifname);
+		} else if (port) {
+			LIST_FOREACH(agent, &port->agent_head, entry) {
+				LLDPAD_DBG("%s: calling ifdown for agent %p.\n",
+					   __func__, agent);
+				LIST_FOREACH(np, &lldp_head, lldp) {
+					ops = np->ops;
+					if (ops->lldp_mod_ifdown)
+						ops->lldp_mod_ifdown(ifname,
+								     agent);
+				}
+			}
+			set_lldp_port_enable(ifname, 0);
 		}
 		p++;
 	}
 
-err_out:
 	if_freenameindex(nameidx);
 	return;
 error_out:
@@ -169,8 +193,7 @@ error_out:
 
 void create_default_cfg_file(void)
 {
-	if (cfg_loaded || init_cfg())
-		config_write_file(&lldpad_cfg, cfg_file_name);
+	config_write_file(&lldpad_cfg, cfg_file_name);
 }
 
 /* check for existence of cfg file.  If it does not exist,
@@ -187,22 +210,20 @@ int check_cfg_file(void)
 
 	if (access(cfg_file_name, R_OK | W_OK)) {
 		if (access(cfg_file_name, F_OK)) {
-			printf("config file failed to load, ");
-			printf("create a new file.\n");
+			LLDPAD_INFO("config file failed to load, ");
+			LLDPAD_INFO("create a new file.\n");
 			fd = open(cfg_file_name,
 				O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
 			if (fd < 0) {
 				retval = errno;
-				log_message(MSG_ERR_SERVICE_START_FAILURE,
-					"error creating %s", cfg_file_name);
+				LLDPAD_ERR("error creating %s !\n", cfg_file_name);
 			} else {
 				close(fd);
 				create_default_cfg_file();
 			}
 		} else {
 			retval = errno;
-			log_message(MSG_ERR_SERVICE_START_FAILURE,
-				"%s is not readable and writeable",
+			LLDPAD_ERR("%s is not readable and writeable",
 				cfg_file_name);
 		}
 	}
@@ -250,18 +271,9 @@ static bool check_priority(int priority_setting)
 		return true;
 }
 
-static bool check_double(double double_setting)
-{
-	if (double_setting < 0 )
-		return false;
-	else
-		return true;
-}
-
 int get_int_config(config_setting_t *s, char *attr, int int_type,
 				int *result)
 {
-	char buf[64];
 	config_setting_t *setting = NULL;
 	int rval = false;
 
@@ -286,10 +298,8 @@ int get_int_config(config_setting_t *s, char *attr, int int_type,
 		}
 	}
 
-	if (!rval) {
-		snprintf(buf, sizeof(buf), "invalid value for %s", attr);
-		log_message(MSG_ERR_DCB_INVALID_CONFIG_FILE, "%s", buf);
-	}
+	if (!rval)
+		LLDPAD_ERR("invalid value for %s", attr);
 
 	return rval;
 }
@@ -297,7 +307,6 @@ int get_int_config(config_setting_t *s, char *attr, int int_type,
 int get_array_config(config_setting_t *s, char *attr, int int_type,
 			int *result)
 {
-	char buf[64];
 	config_setting_t *setting = NULL;
 	config_setting_t *setting_value = NULL;
 	int rval = false;
@@ -333,10 +342,8 @@ int get_array_config(config_setting_t *s, char *attr, int int_type,
 		}
 	}
 
-	if (!rval) {
-		snprintf(buf, sizeof(buf), "invalid setting for %s", attr);
-		log_message(MSG_ERR_DCB_INVALID_CONFIG_FILE, "%s", buf);
-	}
+	if (!rval)
+		LLDPAD_ERR("invalid setting for %s", attr);
 
 	return rval;
 }
@@ -345,34 +352,43 @@ void init_ports(void)
 {
 	struct lldp_module *np;
 	struct if_nameindex *nameidx, *p;
+	struct port *port;
+	struct lldp_agent *agent;
 
 	nameidx = if_nameindex();
 	if (nameidx == NULL) {
-		TRACE("error calling if_nameindex()\n")
+		LLDPAD_DBG("error calling if_nameindex()\n");
 		return;
 	}
 
 	p = nameidx;
 	while (p->if_index != 0) {
-		if (!init_cfg())
-			break;
+		int valid = is_valid_lldp_device(p->if_name);
 
-		if (is_loopback(p->if_name)) {
-			;
-		} else if (is_bond(p->if_name)) {
-			if (add_bond_port(p->if_name) < 0)
-				syslog(LOG_ERR, "failed to register port %s",
-					p->if_name);
-		} else if (is_vlan(p->if_name)) {
-			;
-		} else if (is_bridge(p->if_name)) {
-			; /* ignore bridge device */
+		if (!valid) {
+			p++;
+			continue;
+		}
+
+		port = add_port(p->if_name);
+
+		if (port == NULL) {
+			LLDPAD_ERR("%s: Error adding device %s\n",
+				     __func__, p->if_name);
 		} else if (check_link_status(p->if_name)) {
-			add_adapter(p->if_name);
+			lldp_add_agent(p->if_name, NEAREST_BRIDGE);
+			lldp_add_agent(p->if_name, NEAREST_NONTPMR_BRIDGE);
+			lldp_add_agent(p->if_name, NEAREST_CUSTOMER_BRIDGE);
 
-			LIST_FOREACH(np, &lldp_head, lldp)
-				np->ops->lldp_mod_ifup(p->if_name);
-			set_lldp_port_enable_state(p->if_name, 1);
+			LIST_FOREACH(agent, &port->agent_head, entry) {
+				LLDPAD_DBG("%s: calling ifup for agent %p.\n",
+					   __func__, agent);
+				LIST_FOREACH(np, &lldp_head, lldp) {
+					if (np->ops->lldp_mod_ifup)
+						np->ops->lldp_mod_ifup(p->if_name, agent);
+				}
+			}
+			set_lldp_port_enable(p->if_name, 1);
 		}
 		p++;
 	}
@@ -380,47 +396,56 @@ void init_ports(void)
 	if_freenameindex(nameidx);
 }
 
-static int set_config_value(config_setting_t *setting, void *value, int type)
+static int
+set_config_value(config_setting_t *setting, union cfg_set v, int type)
 {
+	config_int_t tmp_int;
+
 	switch (type) {
 	case CONFIG_TYPE_INT:
-		return config_setting_set_int(setting, *((long *)value));
+		tmp_int = (config_int_t)*v.pint;
+		return config_setting_set_int(setting, tmp_int);
 	case CONFIG_TYPE_INT64:
-		return config_setting_set_int64(setting, *((long long *)value));
+		return config_setting_set_int64(setting, *v.p64);
 	case CONFIG_TYPE_FLOAT:
-		return config_setting_set_float(setting, *((double *)value));
+		return config_setting_set_float(setting, *v.pfloat);
 	case CONFIG_TYPE_STRING:
-		return config_setting_set_string(setting,
-						*((const char **)value));
+		return config_setting_set_string(setting, *v.ppchar);
 	case CONFIG_TYPE_BOOL:
-		return config_setting_set_bool(setting, *((int *)value));
+		return config_setting_set_bool(setting, *v.pint);
 	default:
 		return CONFIG_FALSE;
 	}
 }
 
-static int lookup_config_value(char *path, void *value, int type)
+static int lookup_config_value(char *path, union cfg_get v, int type)
 {
+	config_int_t tmp_int;
+	int rc;
+
 	switch (type) {
 	case CONFIG_TYPE_INT:
-		return config_lookup_int(&lldpad_cfg, path, (long *)value);
+		rc = config_lookup_int(&lldpad_cfg, path, &tmp_int);
+		*v.pint = (int)tmp_int;
+		return rc;
 	case CONFIG_TYPE_INT64:
-		return config_lookup_int64(&lldpad_cfg, path, (long long *)value);
+		return config_lookup_int64(&lldpad_cfg, path, v.p64);
 	case CONFIG_TYPE_FLOAT:
-		return config_lookup_float(&lldpad_cfg, path, (double *)value);
+		return config_lookup_float(&lldpad_cfg, path, v.pfloat);
 	case CONFIG_TYPE_STRING:
-		return config_lookup_string(&lldpad_cfg, path,
-					   (const char **)value);
+		return config_lookup_string(&lldpad_cfg, path, v.ppchar);
 	case CONFIG_TYPE_BOOL:
-		return config_lookup_bool(&lldpad_cfg, path, (int *)value);
+		return config_lookup_bool(&lldpad_cfg, path, v.pint);
 	default:
 		return CONFIG_FALSE;
 	}
 }
 
+
 /*
  * get_config_setting - get the setting from the given config file path by type
  * @ifname: interface name
+ * @agenttype: type of agent this needs to be retrieved from
  * @path: relative to LLDP_COMMON or ifname section of LLDP configuration.
  * @value: pointer to the value to be retrieved
  * @type: libconfig value types
@@ -429,35 +454,70 @@ static int lookup_config_value(char *path, void *value, int type)
  *
  * This function assumes init_cfg() has been called.
  */
-int get_config_setting(const char *ifname, char *path, void *value, int type)
+int get_config_setting(const char *ifname, int agenttype, char *path,
+		       union cfg_get v, int type)
 {
 	char p[1024];
 	int rval = CONFIG_FALSE;
+	const char *section = agent_type2section(agenttype);
 
-	/* look for setting in ifname area first */
+	/* look for setting in section->ifname area first */
 	if (ifname) {
 		snprintf(p, sizeof(p), "%s.%s.%s",
-			 LLDP_SETTING, ifname, path);
-		rval = lookup_config_value(p, value, type);
+			 section, ifname, path);
+		rval = lookup_config_value(p, v, type);
 	}
 
-	/* if not found look for setting in common area */
+	/* if not found look for setting in section->common area */
 	if (rval == CONFIG_FALSE) {
 		snprintf(p, sizeof(p), "%s.%s.%s",
-			 LLDP_SETTING, LLDP_COMMON, path);
-		rval = lookup_config_value(p, value, type);
+			 section, LLDP_COMMON, path);
+		rval = lookup_config_value(p, v, type);
 	}
+
+	return (rval == CONFIG_FALSE) ? cmd_failed : cmd_success;
+}
+
+int remove_config_setting(const char *ifname, int agenttype, char *parent,
+			  char *name)
+{
+	char p[1024];
+	int rval = CONFIG_FALSE;
+	config_setting_t *setting = NULL;
+	const char *section = agent_type2section(agenttype);
+
+	/* look for setting in section->ifname area first */
+	if (ifname) {
+		snprintf(p, sizeof(p), "%s.%s.%s",
+			 section, ifname, parent);
+		setting = config_lookup(&lldpad_cfg, p);
+	}
+
+	/* if not found look for setting in section->common area */
+	if (setting == NULL) {
+		snprintf(p, sizeof(p), "%s.%s.%s",
+			 section, LLDP_COMMON, parent);
+		setting = config_lookup(&lldpad_cfg, p);
+	}
+
+	if (setting != NULL) {
+		rval = config_setting_remove(setting, name);
+		if ((rval == CONFIG_TRUE) &&
+			!config_write_file(&lldpad_cfg, cfg_file_name)) {
+			LLDPAD_DBG("config write failed\n");
+			rval = CONFIG_FALSE;
+		}
+	}
+
 	return (rval == CONFIG_FALSE) ? cmd_failed : cmd_success;
 }
 
 /* calling get_config_setting() w/ init_cfg()/destroy_cfg() */
-int get_cfg(const char *ifname, char *path, void *value, int type)
+int get_cfg(const char *ifname, int agenttype, char *path, union cfg_get v,
+	    int type)
 {
-	int rval = cmd_failed;
-
-	if (init_cfg()) {
-		rval = get_config_setting(ifname, path, value, type);
-	}
+	int rval;
+	rval = get_config_setting(ifname, agenttype, path, v, type);
 	return rval;
 }
 
@@ -512,37 +572,40 @@ config_setting_t *find_or_create_setting(char *p, int type)
  *
  * This function assumes init_cfg() has been called.
  */
-int set_config_setting(const char *ifname, char *path, void *value, int type)
+int set_config_setting(const char *ifname, int agenttype, char *path,
+		       union cfg_set v, int type)
 {
 	config_setting_t *setting = NULL;
 	char p[1024];
 	int rval = cmd_success;
+	const char *section = agent_type2section(agenttype);
+
+	LLDPAD_DBG("%s(%i): \n", __func__, __LINE__);
 
 	if (strlen(ifname))
 		snprintf(p, sizeof(p), "%s.%s.%s",
-			 LLDP_SETTING, ifname, path);
+			 section, ifname, path);
 	else
 		snprintf(p, sizeof(p), "%s.%s.%s",
-			 LLDP_SETTING, LLDP_COMMON, path);
+			 section, LLDP_COMMON, path);
 	setting = find_or_create_setting(p, type);
 
 	if (setting) {
-		if (!set_config_value(setting, value, type)) {
+		if (!set_config_value(setting, v, type)) {
 			rval = cmd_failed;
 		} else if (!config_write_file(&lldpad_cfg, cfg_file_name)) {
-			printf("config write failed\n");
+			LLDPAD_DBG("config write failed\n");
 			rval = cmd_failed;
 		}
 	}
 	return rval;
 }
 
-int set_cfg(const char *ifname, char *path, void *value, int type)
+int set_cfg(const char *ifname, int agenttype, char *path, union cfg_set v,
+	    int type)
 {
 	int rval = cmd_failed;
-	if (init_cfg()) {
-		rval = set_config_setting(ifname, path, value, type);
-	}
+	rval = set_config_setting(ifname, agenttype, path, v, type);
 	return rval;
 }
 
@@ -558,140 +621,156 @@ int set_cfg(const char *ifname, char *path, void *value, int type)
  *
  * Note: must have called init_cfg() before calling this.
  */
-int get_config_tlvfield(const char *ifname, u32 tlvid, const char *field, void *val, int type)
-{
-	int rc;
-	char path[256];
-
-	memset(path, 0, sizeof(path));
-	rc = snprintf(path, sizeof(path), "tlvid%08x.%s", tlvid, field);
-	if (get_config_setting(ifname, path, val, type)) {
-		rc = EIO;
-		goto out_err;
-	}
-	rc = 0;
-
-out_err:
-	return rc;
-}
-
-/* libconfig uses long for int */
-int get_config_tlvfield_int(const char *ifname, u32 tlvid, const char *field, int *value)
-{
-	int rc = 0;
-	long int val = (long int) *value;
-
-	rc = get_config_tlvfield(ifname, tlvid, field, &val, CONFIG_TYPE_INT);
-	*value = (int) val;
-	return rc;
-}
-
-int get_config_tlvfield_bool(const char *ifname, u32 tlvid, const char *field, int *value)
-{
-	return get_config_tlvfield(ifname, tlvid, field, value, CONFIG_TYPE_BOOL);
-}
-
-int get_config_tlvfield_bin(const char *ifname, u32 tlvid, const char *field, void *value, size_t size)
-{
-	int rc = 0;
-	char *str = NULL;
-
-	rc = get_config_tlvfield(ifname, tlvid, field, (void *)&str, CONFIG_TYPE_STRING);
-	if ((rc == 0) && (str != NULL))
-		rc = hexstr2bin(str, value, size);
-	return rc;
-}
-
-int get_config_tlvfield_str(const char *ifname, u32 tlvid, const char *field, void *value, size_t size)
-{
-	int rc = 0;
-	char *str = NULL;
-
-	rc = get_config_tlvfield(ifname, tlvid, field, (void *)&str, CONFIG_TYPE_STRING);
-	if ((rc == 0) && (str != NULL))
-		strncpy(value, str, size);
-	return rc;
-}
-
-int get_config_tlvinfo_bin(const char *ifname, u32 tlvid, void *value, size_t size)
-{
-	return	get_config_tlvfield_bin(ifname, tlvid, ARG_TLVINFO, value, size);
-}
-
-int get_config_tlvinfo_str(const char *ifname, u32 tlvid, void *value, size_t size)
-{
-	return	get_config_tlvfield_str(ifname, tlvid, ARG_TLVINFO, value, size);
-}
-
-int set_config_tlvfield(const char *ifname, u32 tlvid, const char *field, void *val, int type)
+int get_config_tlvfield(const char *ifname, int agenttype, u32 tlvid,
+			const char *field, union cfg_get v, int type)
 {
 	char path[256];
 
 	memset(path, 0, sizeof(path));
 	snprintf(path, sizeof(path), "tlvid%08x.%s", tlvid, field);
-	return set_config_setting(ifname, path, val, type);
+	if (get_config_setting(ifname, agenttype, path, v, type))
+		return EIO;
+
+	return 0;
 }
 
-int set_config_tlvfield_str(const char *ifname, u32 tlvid, const char *field, void *value, size_t size)
+int get_config_tlvfield_int(const char *ifname, int agenttype, u32 tlvid,
+			    const char *field, int *val)
 {
-	int rc = EINVAL;
-	char *str = NULL;
+	return get_config_tlvfield(ifname, agenttype, tlvid, field, val,
+				   CONFIG_TYPE_INT);
+}
 
-	if (value) {
-		str = (char *)value;
-		rc = set_config_tlvfield(ifname, tlvid, field, (void *)&str, CONFIG_TYPE_STRING);
-	}
+int get_config_tlvfield_bool(const char *ifname, int agenttype, u32 tlvid,
+			     const char *field, int *val)
+{
+	return get_config_tlvfield(ifname, agenttype, tlvid, field, val,
+				   CONFIG_TYPE_BOOL);
+}
+
+int get_config_tlvfield_bin(const char *ifname, int agenttype, u32 tlvid,
+			    const char *field, void *value, size_t size)
+{
+	int rc;
+	const char *str = NULL;
+
+	rc = get_config_tlvfield(ifname, agenttype, tlvid, field, &str,
+				 CONFIG_TYPE_STRING);
+	if (rc == 0 && str)
+		rc = hexstr2bin(str, value, size);
 	return rc;
 }
 
-int set_config_tlvfield_bin(const char *ifname, u32 tlvid, const char *field, void *value, size_t size)
+int get_config_tlvfield_str(const char *ifname, int agenttype, u32 tlvid,
+			    const char *field, char *value, size_t size)
 {
-	int rc = EINVAL;
-	char *str = NULL;
-	size_t bsize = 0;
+	int rc;
+	const char *str = NULL;
+
+	rc = get_config_tlvfield(ifname, agenttype, tlvid, field, &str,
+				 CONFIG_TYPE_STRING);
+	if (rc == 0 && str)
+		strncpy(value, str, size);
+	return rc;
+}
+
+int get_config_tlvinfo_bin(const char *ifname, int agenttype, u32 tlvid,
+			   void *value, size_t size)
+{
+	return get_config_tlvfield_bin(ifname, agenttype, tlvid, ARG_TLVINFO,
+				       value, size);
+}
+
+int get_config_tlvinfo_str(const char *ifname, int agenttype, u32 tlvid,
+			   char *value, size_t size)
+{
+	return get_config_tlvfield_str(ifname, agenttype, tlvid, ARG_TLVINFO,
+				       value, size);
+}
+
+int set_config_tlvfield(const char *ifname, int agenttype, u32 tlvid,
+			const char *field, union cfg_set v, int type)
+{
+	char path[256];
+
+	memset(path, 0, sizeof(path));
+	snprintf(path, sizeof(path), "tlvid%08x.%s", tlvid, field);
+	return set_config_setting(ifname, agenttype, path, v, type);
+}
+
+int set_config_tlvfield_str(const char *ifname, int agenttype, u32 tlvid,
+			    const char *field, const char *str)
+{
+	if (!str)
+		return EINVAL;
+	return set_config_tlvfield(ifname, agenttype, tlvid, field, &str,
+				   CONFIG_TYPE_STRING);
+}
+
+int set_config_tlvfield_bin(const char *ifname, int agenttype, u32 tlvid,
+			    const char *field, void *value, size_t size)
+{
+	int rc;
+	const size_t bsize = size * 2 + 1; /* 2 char per byte + '\0' */
+	char str[bsize];
+	const char *p;
 
 	if (!value)
-		goto out_err;
+		return EINVAL;
 
-	/* 2 char per byte + '\0' */
-	bsize = size * 2 + 1;
-	str = malloc(bsize);
-	if (!str)
-		goto out_err;
 	memset(str, 0, bsize);
-	rc = bin2hexstr((u8 *)value, size, str, bsize);
+	rc = bin2hexstr(value, size, str, bsize);
 	if (rc)
-		goto out_free;
+		goto out;
 	str[bsize - 1] = '\0';
-	rc = set_config_tlvfield(ifname, tlvid, field, (void *)&str, CONFIG_TYPE_STRING);
-out_free:
-	free(str);
-out_err:
+	p = str;
+	rc = set_config_tlvfield(ifname, agenttype, tlvid, field, &p,
+				 CONFIG_TYPE_STRING);
+out:
 	return rc;
 }
 
-int set_config_tlvinfo_bin(const char *ifname, u32 tlvid, void *value, size_t size)
+int set_config_tlvinfo_bin(const char *ifname, int agenttype, u32 tlvid,
+			   void *value, size_t size)
 {
-	return	set_config_tlvfield_bin(ifname, tlvid, "info", value, size);
+	return set_config_tlvfield_bin(ifname, agenttype, tlvid, "info",
+				       value, size);
 }
 
-int set_config_tlvinfo_str(const char *ifname, u32 tlvid, void *value, size_t size)
+int set_config_tlvinfo_str(const char *ifname, int agenttype, u32 tlvid,
+			   char *value)
 {
-	return	set_config_tlvfield_str(ifname, tlvid, "info", value, size);
+	return set_config_tlvfield_str(ifname, agenttype, tlvid, "info", value);
 }
 
-int set_config_tlvfield_int(const char *ifname, u32 tlvid, const char *field, int *value)
+int set_config_tlvfield_int(const char *ifname, int agenttype, u32 tlvid,
+			    const char *field, int *value)
 {
-	long int val = (long int )*value;
-	return set_config_tlvfield(ifname, tlvid, field, &val, CONFIG_TYPE_INT);
+	return set_config_tlvfield(ifname, agenttype, tlvid, field, value,
+				   CONFIG_TYPE_INT);
 }
 
-int set_config_tlvfield_bool(const char *ifname, u32 tlvid, const char *field, int *value)
+int set_config_tlvfield_bool(const char *ifname, int agenttype, u32 tlvid,
+			     const char *field, int *value)
 {
-	return set_config_tlvfield(ifname, tlvid, field, value, CONFIG_TYPE_BOOL);
+	return set_config_tlvfield(ifname, agenttype, tlvid, field, value,
+				   CONFIG_TYPE_BOOL);
 }
 
-int is_tlv_txenabled(const char *ifname, u32 tlvid)
+int is_tlv_txdisabled(const char *ifname, int agenttype, u32 tlvid)
+{
+	char arg[64];
+	int enabletx = true;
+
+	snprintf(arg, sizeof(arg), "%s%08x.%s", TLVID_PREFIX,
+		 tlvid, ARG_TLVTXENABLE);
+
+	get_config_setting(ifname, agenttype, arg, &enabletx, CONFIG_TYPE_BOOL);
+
+	return !enabletx;
+}
+
+int is_tlv_txenabled(const char *ifname, int agenttype, u32 tlvid)
 {
 	char arg[64];
 	int enabletx = false;
@@ -699,38 +778,43 @@ int is_tlv_txenabled(const char *ifname, u32 tlvid)
 	snprintf(arg, sizeof(arg), "%s%08x.%s", TLVID_PREFIX,
 		 tlvid, ARG_TLVTXENABLE);
 
-	get_config_setting(ifname, arg, (void *)&enabletx, CONFIG_TYPE_BOOL);
+	get_config_setting(ifname, agenttype, arg, &enabletx, CONFIG_TYPE_BOOL);
 
 	return enabletx;
 }
 
-int tlv_enabletx(const char *ifname, u32 tlvid)
+int tlv_enabletx(const char *ifname, int agenttype, u32 tlvid)
 {
 	int enabletx = true;
-	return set_config_tlvfield_bool(ifname, tlvid,
-					ARG_TLVTXENABLE, (void *)&enabletx);
+	return set_config_tlvfield_bool(ifname, agenttype, tlvid,
+					ARG_TLVTXENABLE, &enabletx);
 }
 
-int tlv_disabletx(const char *ifname, u32 tlvid)
+int tlv_disabletx(const char *ifname, int agenttype, u32 tlvid)
 {
 	int enabletx = false;
-	return set_config_tlvfield_bool(ifname, tlvid,
-					ARG_TLVTXENABLE, (void *)&enabletx);
+	return set_config_tlvfield_bool(ifname, agenttype, tlvid,
+					ARG_TLVTXENABLE, &enabletx);
 }
 
-void set_med_devtype(const char *ifname, int devtype)
+void set_med_devtype(const char *ifname, int agenttype, int devtype)
 {
 	if (LLDP_MED_DEVTYPE_INVALID(devtype))
 		return;
-	set_config_tlvfield_int(ifname, TLVID_MED(LLDP_MED_RESERVED),
+	set_config_tlvfield_int(ifname, agenttype, TLVID_MED(LLDP_MED_RESERVED),
 				"devtype", &devtype);
 }
 
-int get_med_devtype(const char *ifname)
+int get_med_devtype(const char *ifname, int agenttype)
 {
-	int devtype;
+	int devtype, err;
+
+	err = get_config_tlvfield_int(ifname, agenttype,
+				      TLVID_MED(LLDP_MED_RESERVED),
+				      "devtype", &devtype);
+
+	if (err)
+		devtype = 0;
 
-	get_config_tlvfield_int(ifname, TLVID_MED(LLDP_MED_RESERVED),
-				"devtype", &devtype);
 	return devtype;
 }
