@@ -1,7 +1,7 @@
 /*******************************************************************************
 
   LLDP Agent Daemon (LLDPAD) Software 
-  Copyright(c) 2007-2010 Intel Corporation.
+  Copyright(c) 2007-2012 Intel Corporation.
 
   Substantially modified from:
   hostapd-0.5.7
@@ -25,13 +25,17 @@
   the file called "COPYING".
 
   Contact Information:
-  e1000-eedc Mailing List <e1000-eedc@lists.sourceforge.net>
-  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497
+  open-lldp Mailing List <lldp-devel@open-lldp.org>
 
 *******************************************************************************/
 
-#include "includes.h"
+#include <stdlib.h>
+#include <stdio.h>
+#include <signal.h>
 #include <dirent.h>
+#include <unistd.h>
+#include <readline/readline.h>
+#include <readline/history.h>
 
 #include "clif.h"
 #include "lldp_dcbx_cmds.h"
@@ -40,6 +44,8 @@
 #include "dcbtool.h"
 #include "version.h"
 
+#define UNUSED __attribute__((__unused__))
+
 static int show_raw;
 
 static const char *cli_version =
@@ -108,8 +114,11 @@ static const char *commands_help =
 "  <gc|go> dcbx                 get configured or operational DCBX versions\n"
 "                               the configured version takes effect on next\n"
 "                               restart of the lldpad service\n"
-"  sc dcbx v:[12]               set the DCBX version to be used after next\n"
-"                               lldpad restart\n"
+"  sc dcbx v:cee                set the DCBX version to be used after next\n"
+"          v:cin                lldpad restart.  Version can be set to:\n"
+"          v:force-cin          cin: fall back to CIN DCBX if non-IEEE peer\n"
+"          v:force-cee          cee: fall back to CEE DCBX if non-IEEE peer\n"
+"                               force-cin, force-cee: do not start with IEEE DCBX\n"
 "Per Port Commands:\n"
 "  gc <ifname> <feature>        get configuration of <feature> on port <ifname>\n"
 "  go <ifname> <feature>        get operational status of <feature>\n"
@@ -125,6 +134,8 @@ static const char *commands_help =
 "     ll:<subtype>              logical link status\n\n"
 "  'subtype' can be:\n"
 "     [0|fcoe]                  FCoE\n\n"
+"     [1|iscsi]                 iSCSI\n\n"
+"     [2|fip]                   FIP\n\n"
 "  'args' can include:\n"
 "     [e:<0|1>]                 controls feature enable\n"
 "     [a:<0|1>]                 controls feature advertise via DCBX\n"
@@ -181,8 +192,6 @@ static const char *commands_help =
 static struct clif *clif_conn;
 static int cli_quit = 0;
 static int cli_attached = 0;
-static const char *clif_iface_dir = CLIF_IFACE_DIR;
-static char *clif_ifname = NULL;
 
 
 static void usage(void)
@@ -211,7 +220,7 @@ void print_raw_message(char *msg, int print)
 		return;
 
 	if (!(print & SHOW_RAW_ONLY)) {
-		switch (msg[0]) {
+		switch (msg[MSG_TYPE]) {
 		case EVENT_MSG:
 			printf("event: ");
 			break;
@@ -237,34 +246,14 @@ int parse_print_message(char *msg, int print)
 	if (print & SHOW_RAW_ONLY)
 		return status;
 
-	if (msg[0] == CMD_RESPONSE)
+	if (msg[MSG_TYPE] == CMD_RESPONSE)
 		print_response(msg, status);
-	else if (msg[0] == EVENT_MSG)
+	else if (msg[MSG_TYPE] == EVENT_MSG)
 		print_event_msg(msg);
 
 	return status;
 }
 
-static struct clif *cli_open_connection(const char *ifname)
-{
-	char *cfile;
-	int flen;
-
-	if (ifname == NULL)
-		return NULL;
-
-	flen = strlen(clif_iface_dir) + strlen(ifname) + 2;
-	cfile = malloc(flen);
-	if (cfile == NULL)
-		return NULL;
-	snprintf(cfile, flen, "%s/%s", clif_iface_dir, ifname);
-
-	clif_conn = clif_open(cfile);
-	free(cfile);
-	return clif_conn;
-}
-
-
 static void cli_close_connection(void)
 {
 	if (clif_conn == NULL)
@@ -279,7 +268,7 @@ static void cli_close_connection(void)
 }
 
 
-static void cli_msg_cb(char *msg, size_t len)
+static void cli_msg_cb(char *msg, UNUSED size_t len)
 {
 	parse_print_message(msg, SHOW_OUTPUT | show_raw);
 }
@@ -327,83 +316,37 @@ inline int clif_command(struct clif *clif, char *cmd, int raw)
 	return _clif_command(clif, cmd, SHOW_OUTPUT | raw);
 }
 
-
-static int cli_cmd_ping(struct clif *clif, int argc, char *argv[], int raw)
+static int cli_cmd_ping(struct clif *clif, UNUSED int argc, UNUSED char *argv[],
+			int raw)
 {
 	return clif_command(clif, "P", raw);
 }
 
-static int cli_cmd_help(struct clif *clif, int argc, char *argv[], int raw)
+static int
+cli_cmd_help(UNUSED struct clif *clif, UNUSED int argc, UNUSED char *argv[],
+	     UNUSED int raw)
 {
 	printf("%s", commands_help);
 	return 0;
 }
 
-
-static int cli_cmd_license(struct clif *clif, int argc, char *argv[], int raw)
+static int
+cli_cmd_license(UNUSED struct clif *clif, UNUSED int argc, UNUSED char *argv[],
+		UNUSED int raw)
 {
 	printf("%s\n\n%s\n", cli_version, cli_full_license);
 	return 0;
 }
 
-
-static int cli_cmd_quit(struct clif *clif, int argc, char *argv[], int raw)
+static int
+cli_cmd_quit(UNUSED struct clif *clif, UNUSED int argc, UNUSED char *argv[],
+	     UNUSED int raw)
 {
 	cli_quit = 1;
 	return 0;
 }
 
 
-static void cli_list_interfaces(struct clif *clif)
-{
-	struct dirent *dent;
-	DIR *dir;
-
-	dir = opendir(clif_iface_dir);
-	if (dir == NULL) {
-		printf("Control interface directory '%s' could not be "
-		       "opened.\n", clif_iface_dir);
-		return;
-	}
-
-	printf("Available interfaces:\n");
-	while ((dent = readdir(dir))) {
-		if (strcmp(dent->d_name, ".") == 0 ||
-		    strcmp(dent->d_name, "..") == 0)
-			continue;
-		printf("%s\n", dent->d_name);
-	}
-	closedir(dir);
-}
-
-
-static int cli_cmd_interface(struct clif *clif, int argc,
-				     char *argv[], int raw)
-{
-	if (argc < 1) {
-		cli_list_interfaces(clif);
-		return 0;
-	}
-
-	cli_close_connection();
-	free(clif_ifname);
-	clif_ifname = strdup(argv[0]);
-
-	if (cli_open_connection(clif_ifname)) {
-		printf("Connected to interface '%s.\n", clif_ifname);
-		if (clif_attach(clif_conn, NULL) == 0)
-			cli_attached = 1;
-		else
-			printf("Warning: Failed to attach to lldpad.\n");
-	} else {
-		printf(
-		     "Could not connect to lldpad interface '%s' - re-trying\n",
-			clif_ifname);
-	}
-	return 0;
-}
-
-
 struct cli_cmd {
 	const char *cmd;
 	int (*handler)(struct clif *clif, int argc, char *argv[], int raw);
@@ -412,7 +355,6 @@ struct cli_cmd {
 static struct cli_cmd cli_commands[] = {
 	{ "ping", cli_cmd_ping },
 	{ "help", cli_cmd_help },
-	{ "interface", cli_cmd_interface },
 	{ "license", cli_cmd_license },
 	{ "quit", cli_cmd_quit },
 	{ NULL, NULL }
@@ -478,31 +420,26 @@ static void cli_recv_pending(struct clif *clif, int in_read)
 	}
 }
 
-
 static void cli_interactive(int raw)
 {
 	const int max_args = 10;
-	char cmd[256], *res, *argv[max_args], *pos;
+	char *cmd, *argv[max_args], *pos;
 	int argc;
 
+	setlinebuf(stdout);
 	printf("\nInteractive mode\n\n");
+	using_history();
+	stifle_history(1000);
 
 	do {
 		cli_recv_pending(clif_conn, 0);
-		printf("> ");
 		alarm(1);
-		res = fgets(cmd, sizeof(cmd), stdin);
+		cmd = readline("> ");
 		alarm(0);
-		if (res == NULL)
+		if (!cmd)
 			break;
-		pos = cmd;
-		while (*pos != '\0') {
-			if (*pos == '\n') {
-				*pos = '\0';
-				break;
-			}
-			pos++;
-		}
+		if (*cmd)
+			add_history(cmd);
 		argc = 0;
 		pos = cmd;
 		for (;;) {
@@ -521,25 +458,24 @@ static void cli_interactive(int raw)
 		}
 		if (argc)
 			request(clif_conn, argc, argv, raw);
+		free(cmd);
 	} while (!cli_quit);
 }
 
-
-static void cli_terminate(int sig)
+static void cli_terminate(UNUSED int sig)
 {
 	cli_close_connection();
 	exit(0);
 }
 
-
-static void cli_alarm(int sig)
+static void cli_alarm(UNUSED int sig)
 {
 	if (clif_conn && _clif_command(clif_conn, "P", SHOW_NO_OUTPUT)) {
 		printf("Connection to lldpad lost - trying to reconnect\n");
 		cli_close_connection();
 	}
 	if (!clif_conn) {
-		clif_conn = cli_open_connection(clif_ifname);
+		clif_conn = clif_open();
 		if (clif_conn) {
 			printf("Connection to lldpad re-established\n");
 			if (clif_attach(clif_conn, NULL) == 0)
@@ -603,21 +539,7 @@ int main(int argc, char *argv[])
 	}
 
 	for (;;) {
-		if (clif_ifname == NULL) {
-			struct dirent *dent;
-			DIR *dir = opendir(clif_iface_dir);
-			if (dir) {
-				while ((dent = readdir(dir))) {
-					if (strcmp(dent->d_name, ".") == 0 ||
-					    strcmp(dent->d_name, "..") == 0)
-						continue;
-					clif_ifname = strdup(dent->d_name);
-					break;
-				}
-				closedir(dir);
-			}
-		}
-		clif_conn = cli_open_connection(clif_ifname);
+		clif_conn = clif_open();
 		if (clif_conn) {
 			if (warning_displayed)
 				printf("Connection established.\n");
@@ -650,7 +572,6 @@ int main(int argc, char *argv[])
 	} else
 		ret = request(clif_conn, argc - optind, &argv[optind], raw);
 
-	free(clif_ifname);
 	cli_close_connection();
 	return ret;
 }
