@@ -1,7 +1,7 @@
 /*******************************************************************************
 
   LLDP Agent Daemon (LLDPAD) Software
-  Copyright(c) 2007-2010 Intel Corporation.
+  Copyright(c) 2007-2012 Intel Corporation.
 
   This program is free software; you can redistribute it and/or modify it
   under the terms and conditions of the GNU General Public License,
@@ -20,15 +20,30 @@
   the file called "COPYING".
 
   Contact Information:
-  e1000-eedc Mailing List <e1000-eedc@lists.sourceforge.net>
-  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497
+  open-lldp Mailing List <lldp-devel@open-lldp.org>
 
 *******************************************************************************/
 
+#ifndef _LLDP_RTNL_H
 #define _LLDP_RTNL_H
-#include <linux/netlink.h>
 
-typedef int rtnl_handler(struct nlmsghdr *nh, void *arg);
+#ifndef IFNAMSIZ
+#define IFNAMSIZ 16
+#endif
 
+#include "include/linux/netlink.h"
+
+/*
+ * Helper functions to construct a netlink message.
+ */
+void mynla_nest_end(struct nlmsghdr *, struct nlattr *);
+struct nlattr *mynla_nest_start(struct nlmsghdr *, int);
+void mynla_put(struct nlmsghdr *, int, size_t, void *);
+void mynla_put_u16(struct nlmsghdr *, int, __u16);
+void mynla_put_u32(struct nlmsghdr *, int, __u32);
+
+int get_operstate(char *ifname);
 int set_operstate(char *ifname, __u8 operstate);
-int set_linkmode(char *ifname, __u8 linkmode);
+int set_linkmode(const char *ifname, __u8 linkmode);
+
+#endif
