@@ -20,29 +20,30 @@
   the file called "COPYING".
 
   Contact Information:
-  e1000-eedc Mailing List <e1000-eedc@lists.sourceforge.net>
-  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497
+  open-lldp Mailing List <lldp-devel@open-lldp.org>
 
 *******************************************************************************/
 
-#include "common.h"
 #include <stdio.h>
 #include <syslog.h>
 #include <sys/stat.h>
+#include <stdlib.h>
 #include "lldp.h"
 #include "lldpad.h"
 #include "dcb_events.h"
 #include "lldp_mand_clif.h"
 #include "lldp_dcbx_cmds.h"
 #include "lldp_dcbx_cfg.h"
-#include "clif_msgs.h"
+#include "lldpad_status.h"
 #include "dcb_protocol.h"
 #include "lldp/ports.h"
 #include "config.h"
-#include "drv_cfg.h"
+#include "lldp_dcbx_nl.h"
 #include "lldp/states.h"
 #include "lldp_dcbx.h"
 #include "lldp_rtnl.h"
+#include "messages.h"
+#include "lldp_util.h"
 
 extern u8 gdcbx_subtype;
 
@@ -50,10 +51,10 @@ static char *hexlist = "0123456789abcdef";
 static char *hexlistcaps = "0123456789ABCDEF";
 
 static int get_dcb_state(char *port_id, char *rbuf);
-static dcb_result set_dcb_state(char *port_id, char *ibuf, int ilen);
+static cmd_status set_dcb_state(char *port_id, char *ibuf, int ilen);
 static int get_dcbx_config(u8 cmd, char *rbuf);
-static dcb_result set_dcbx_config(char *ibuf, int ilen);
-static dcb_result get_pg_data(pg_attribs *pg_data, int cmd, char *port_id,
+static cmd_status set_dcbx_config(char *ibuf, int ilen);
+static cmd_status get_pg_data(pg_attribs *pg_data, int cmd, char *port_id,
 		int dcbx_st, char *rbuf);
 static int set_pg_config(pg_attribs *pg_data, char *port_id, char *ibuf,
 		int ilen);
@@ -64,28 +65,33 @@ static int get_pfc_data(pfc_attribs *pfc_data, int cmd, char *port_id,
 static int set_app_config(app_attribs *app_data, char *port_id, u32 subtype,
 		char *ibuf, int ilen);
 static int get_app_data(app_attribs *app_data, int cmd, char *port_id,
-		u32 subtype, int dcbx_st, char *rbuf);
+		u32 subtype, char *rbuf);
 static int set_llink_config(llink_attribs *app_data, char *port_id, u32 subtype,
 		char *ibuf, int ilen);
 static int get_llink_data(llink_attribs *llink_data, int cmd, char *port_id,
-		u32 subtype, int dcbx_st, char *rbuf);
-static dcb_result get_bwg_desc(char *port_id, char *ibuf, int ilen, char *rbuf);
-static dcb_result set_bwg_desc(char *port_id, char *ibuf, int ilen);
+		u32 subtype, char *rbuf);
+static cmd_status get_bwg_desc(char *port_id, char *ibuf, int ilen, char *rbuf);
+static cmd_status set_bwg_desc(char *port_id, char *ibuf, int ilen);
 static void set_protocol_data(feature_protocol_attribs *protocol, char *ifname,
-		char *ibuf, int plen);
-static dcb_result get_cmd_protocol_data(feature_protocol_attribs *protocol,
+		char *ibuf, int plen, int agenttype);
+static cmd_status get_cmd_protocol_data(feature_protocol_attribs *protocol,
 	u8 cmd, char *rbuf);
 
-static int get_arg_tlvtxenable(struct cmd *, char *, char *, char *);
-static int set_arg_tlvtxenable(struct cmd *, char *, char *, char *);
+static int get_arg_tlvtxenable(struct cmd *, char *, char *, char *, int);
+static int set_arg_tlvtxenable(struct cmd *, char *, char *, char *, int);
+static int test_arg_tlvtxenable(struct cmd *, char *, char *, char *, int);
 
 static struct arg_handlers arg_handlers[] = {
-	{ ARG_TLVTXENABLE, get_arg_tlvtxenable, set_arg_tlvtxenable },
-	{ NULL }
+	{	.arg = ARG_TLVTXENABLE, .arg_class = TLV_ARG,
+		.handle_get = get_arg_tlvtxenable,
+		.handle_set = set_arg_tlvtxenable,
+		.handle_test = test_arg_tlvtxenable, },
+	{	.arg = 0 }
 };
 
-static int get_arg_tlvtxenable(struct cmd *cmd, char *arg, char *argvalue,
-			       char *obuf)
+static int
+get_arg_tlvtxenable(struct cmd *cmd, char *arg, UNUSED char *argvalue,
+		    char *obuf, int obuf_len)
 {
 	int value;
 	char *s;
@@ -99,8 +105,8 @@ static int get_arg_tlvtxenable(struct cmd *cmd, char *arg, char *argvalue,
 	case (OUI_CEE_DCBX << 8) | 2:
 		snprintf(arg_path, sizeof(arg_path), "%s%08x.%s",
 			 TLVID_PREFIX, cmd->tlvid, arg);
-		
-		if (get_config_setting(cmd->ifname, arg_path, (void *)&value,
+
+		if (get_config_setting(cmd->ifname, cmd->type, arg_path, &value,
 					CONFIG_TYPE_BOOL))
 			value = false;
 		break;
@@ -115,54 +121,54 @@ static int get_arg_tlvtxenable(struct cmd *cmd, char *arg, char *argvalue,
 	else
 		s = VAL_NO;
 	
-	sprintf(obuf, "%02x%s%04x%s", (unsigned int)strlen(arg), arg,
-		(unsigned int)strlen(s), s);
+	snprintf(obuf, obuf_len, "%02zx%s%04zx%s", strlen(arg), arg,
+		 strlen(s), s);
 
 	return cmd_success;
 }
 
-void dont_advertise_dcbx_all(char *ifname)
+void dont_advertise_dcbx_all(char *ifname, bool ad)
 {
-	int i;
+	int i, is_pfc;
 	pfc_attribs pfc_data;
 	pg_attribs pg_data;
 	app_attribs app_data;
 	llink_attribs llink_data;
 	u32 event_flag = 0;
 
-	if (get_pg(ifname, &pg_data) == dcb_success &&
-	    pg_data.protocol.Advertise) {
-		pg_data.protocol.Advertise = 0;
-		put_pg(ifname, &pg_data);
+	is_pfc = get_pfc(ifname, &pfc_data);
+
+	if (get_pg(ifname, &pg_data) == cmd_success) {
+		pg_data.protocol.Advertise = ad;
+		put_pg(ifname, &pg_data, &pfc_data);
 		event_flag |= DCB_LOCAL_CHANGE_PG;
 	}
 
-	if (get_pfc(ifname, &pfc_data) == dcb_success &&
-	    pfc_data.protocol.Advertise) {
-		pfc_data.protocol.Advertise = 0;
+	if (is_pfc == cmd_success) {
+		pfc_data.protocol.Advertise = ad;
 		put_pfc(ifname, &pfc_data);
 		event_flag |= DCB_LOCAL_CHANGE_PFC;
 	}
 
 	for (i = 0; i < DCB_MAX_APPTLV ; i++) {
-		if (get_app(ifname, (u32)i, &app_data) == dcb_success &&
-		    app_data.protocol.Advertise) {
-			app_data.protocol.Advertise = 0;
+		if (get_app(ifname, (u32)i, &app_data) == cmd_success) {
+			app_data.protocol.Advertise = ad;
 			put_app(ifname, (u32)i, &app_data);
-			event_flag |= DCB_LOCAL_CHANGE_APPTLV;
+			event_flag |= DCB_LOCAL_CHANGE_APPTLV(i);
 		}
+	}
 
-		if (get_llink(ifname, (u32)i, &llink_data) == dcb_success &&
-		    llink_data.protocol.Advertise) {
-			llink_data.protocol.Advertise = 0;
+	for (i = 0; i < DCB_MAX_LLKTLV ; i++) {
+		if (get_llink(ifname, (u32)i, &llink_data) == cmd_success) {
+			llink_data.protocol.Advertise = ad;
 			put_llink(ifname, (u32)i, &llink_data);
 			event_flag |= DCB_LOCAL_CHANGE_LLINK;
 		}
 	}
 }
 
-static int set_arg_tlvtxenable(struct cmd *cmd, char *arg, char *argvalue,
-			       char *obuf)
+static int _set_arg_tlvtxenable(struct cmd *cmd, char *arg, char *argvalue,
+			       char *obuf, int obuf_len, bool test)
 {
 	int value;
 	int current_value;
@@ -188,27 +194,41 @@ static int set_arg_tlvtxenable(struct cmd *cmd, char *arg, char *argvalue,
 	else
 		return cmd_invalid;
 
-	current_value = is_tlv_txenabled(cmd->ifname, cmd->tlvid);
+	if (test)
+		return cmd_success;
+
+	current_value = is_tlv_txenabled(cmd->ifname, cmd->type, cmd->tlvid);
+
+	snprintf(obuf, obuf_len, "enabled = %s\n", value ? "yes" : "no");
+
 	if (current_value == value)
 		return cmd_success;
 
 	snprintf(arg_path, sizeof(arg_path), "%s%08x.%s", TLVID_PREFIX,
 		 cmd->tlvid, arg);
 
-	if (set_cfg(cmd->ifname, arg_path, (void *)&value, CONFIG_TYPE_BOOL))
+	if (set_cfg(cmd->ifname, cmd->type, arg_path, &value,
+		    CONFIG_TYPE_BOOL))
 		return cmd_failed;
 
-	/* if DCBX tlv is changed to disabled, then ensure all DCBX features
-	 * are set to 'not advertise'
-	 */
-	if (!value)
-		dont_advertise_dcbx_all(cmd->ifname);
-
-	somethingChangedLocal(cmd->ifname);
+	dont_advertise_dcbx_all(cmd->ifname, value);
+	somethingChangedLocal(cmd->ifname, cmd->type);
 
 	return cmd_success;
 }
 
+static int set_arg_tlvtxenable(struct cmd *cmd, char *arg, char *argvalue,
+			       char *obuf, int obuf_len)
+{
+	return _set_arg_tlvtxenable(cmd, arg, argvalue, obuf, obuf_len, false);
+}
+
+static int test_arg_tlvtxenable(struct cmd *cmd, char *arg, char *argvalue,
+			       char *obuf, int obuf_len)
+{
+	return _set_arg_tlvtxenable(cmd, arg, argvalue, obuf, obuf_len, true);
+}
+
 struct arg_handlers *dcbx_get_arg_handlers()
 {
 	return &arg_handlers[0];
@@ -217,44 +237,33 @@ struct arg_handlers *dcbx_get_arg_handlers()
 static int get_dcb_state(char *port_id, char *rbuf)
 {
 	int state = 0;
-	dcb_result rval = dcb_success;
+	cmd_status rval = cmd_success;
 	
 	if (get_hw_state(port_id, &state))
-		rval = dcb_failed;
+		rval = cmd_failed;
 	else
 		sprintf(rbuf, "%01x", state);
 
 	return (rval);
 }
 
-static dcb_result set_dcb_state(char *port_id, char *ibuf, int ilen)
+static cmd_status set_dcb_state(char *port_id, char *ibuf, int ilen)
 {
 	bool state;
-	int tmp;
 	int off;
 	int plen;
-	dcb_result rval = dcb_success;
+	cmd_status rval = cmd_success;
 
 	plen=strlen(port_id);
 	off = DCB_PORT_OFF + plen;
 
 	if (ilen == (off + CFG_DCB_DLEN)) {
 		state = (*(ibuf+off+DCB_STATE)) ^ '0';
-		if (state == false || state == true) {
-			/* if get_hw_state fails, then don't bother
-			 * trying to set the state.
-			*/
-			if (get_hw_state(port_id, &tmp) ||
-				set_hw_state(port_id, state))
-				rval = dcb_failed;
-			else
-				rval = save_dcb_enable_state(port_id, state);
-		} else {
-			rval = dcb_bad_params;
-		}
+		set_hw_state(port_id, state);
+		rval = save_dcb_enable_state(port_id, state);
 	} else {
 		printf("error - setcommand has invalid argument length\n");
-		rval = dcb_bad_params;
+		rval = cmd_bad_params;
 	}
 
 	return rval;
@@ -262,27 +271,27 @@ static dcb_result set_dcb_state(char *port_id, char *ibuf, int ilen)
 
 static int get_dcbx_config(u8 cmd, char *rbuf)
 {
-	dcb_result rval = dcb_success;
+	cmd_status rval = cmd_success;
 	int dcbx_version;
 	
 	if (cmd == CMD_GET_CONFIG) {
 		if (!get_dcbx_version(&dcbx_version))
-			rval = dcb_failed;
+			rval = cmd_failed;
 	} else {
 		dcbx_version = gdcbx_subtype;
 	}
 
-	if (rval == dcb_success)
+	if (rval == cmd_success)
 		sprintf(rbuf, "%01x", dcbx_version);
 
 	return (rval);
 }
 
-static dcb_result set_dcbx_config(char *ibuf, int ilen)
+static cmd_status set_dcbx_config(char *ibuf, int ilen)
 {
 	int version;
 	int off;
-	dcb_result rval = dcb_success;
+	cmd_status rval = cmd_success;
 
 	off = DCBX_CFG_OFF;
 
@@ -291,39 +300,41 @@ static dcb_result set_dcbx_config(char *ibuf, int ilen)
 		switch (version) {
 		case dcbx_subtype1:
 		case dcbx_subtype2:
+		case dcbx_force_subtype1:
+		case dcbx_force_subtype2:
 			rval = save_dcbx_version(version);
 			break;
 		default:
-			rval = dcb_bad_params;
+			rval = cmd_bad_params;
 			break;
 		}
 	} else {
 		printf("error - setcommand has invalid argument length\n");
-		rval = dcb_bad_params;
+		rval = cmd_bad_params;
 	}
 
 	return rval;
 }
 
-static dcb_result get_bwg_desc(char *port_id, char *ibuf, int ilen, char *rbuf)
+static cmd_status get_bwg_desc(char *port_id, char *ibuf, int ilen, char *rbuf)
 {
 	u8 pgid;
 	int plen;
 	char *desc_str = NULL;
-	dcb_result rval = dcb_success;
+	cmd_status rval = cmd_success;
 
 	plen = strlen(port_id);
 
 	if (ilen !=  DCB_PORT_OFF + plen + PG_DESC_GET_DLEN) {
-		rval = dcb_bad_params;
+		rval = cmd_bad_params;
 	} else {
 		pgid = *(ibuf + DCB_PORT_OFF + plen + PG_DESC_PGID) & 0x0f;
 
 		if (pgid < 1 || pgid > 8) {
-			rval = dcb_bad_params;
+			rval = cmd_bad_params;
 		} else {
 			rval = get_bwg_descrpt(port_id, pgid-1, &desc_str);
-			if (rval == dcb_success && desc_str) {
+			if (rval == cmd_success && desc_str) {
 				sprintf(rbuf, "%01x%02x%s", pgid,
 					(unsigned int) strlen(desc_str),
 					desc_str);
@@ -335,12 +346,12 @@ static dcb_result get_bwg_desc(char *port_id, char *ibuf, int ilen, char *rbuf)
 	return rval;
 }
 
-static dcb_result set_bwg_desc(char *port_id, char *ibuf, int ilen)
+static cmd_status set_bwg_desc(char *port_id, char *ibuf, int ilen)
 {
 	u8 pgid;
 	u8 desc_len = 0;
 	int plen;
-	dcb_result rval = dcb_success;
+	cmd_status rval = cmd_success;
 
 	plen = strlen(port_id);
 	if (ilen >=  DCB_PORT_OFF + plen + PG_DESC_SET_DLEN) {
@@ -350,32 +361,90 @@ static dcb_result set_bwg_desc(char *port_id, char *ibuf, int ilen)
 	} else {
 		printf("ilen[%d] < %d\n", ilen, DCB_PORT_OFF + plen +
 			PG_DESC_SET_DLEN);
-		return dcb_bad_params;
+		return cmd_bad_params;
 	}
 
 	if (ilen ==  DCB_PORT_OFF + plen + PG_DESC_SET_DLEN + desc_len) {
 		if (pgid < 1 || pgid > 8 || desc_len > 100) {
 			printf("pgid = %d, desc_len = %d\n", pgid, desc_len);
-			rval = dcb_bad_params;
+			rval = cmd_bad_params;
 		} else {
-			if (init_cfg()) {
-				rval = put_bwg_descrpt(port_id, pgid-1,
-					ibuf+DCB_PORT_OFF+plen+PG_DESC_DATA);
-			} else {
-				rval = dcb_failed;
-			}
+			rval = put_bwg_descrpt(port_id, pgid-1,
+				ibuf+DCB_PORT_OFF+plen+PG_DESC_DATA);
 		}
 	} else {
 		printf("ilen[%d] != %d\n", ilen, DCB_PORT_OFF + plen +
 			PG_DESC_SET_DLEN + desc_len);
-		rval = dcb_bad_params;
+		rval = cmd_bad_params;
 	}
 
 	return rval;
 }
 
+static void set_app_protocol_data(char *ifname, char *ibuf, int plen, int ilen,
+				  int subtype, int agenttype)
+{
+	u8 aflag, eflag, wflag;
+	int status, last, i;
+	app_attribs app_data;
+	feature_protocol_attribs *protocol;
+
+	status = get_app(ifname, (u32)subtype, &app_data);
+	if (status != cmd_success) {
+		printf("%s %s: error[%d] getting APP data.\n",
+			__func__, ifname, status);
+		return;
+	}
+
+	protocol = &app_data.protocol;
+	last = protocol->Advertise;
+
+	aflag = *(ibuf+DCB_PORT_OFF+plen+CFG_ADVERTISE);
+	if (aflag == '0' || aflag == '1')
+		protocol->Advertise = aflag & 0x01;
+
+	status = set_app_config(&app_data,
+				ifname,
+				(u32)subtype,
+				ibuf,
+				ilen);
+
+	if (status != cmd_success)
+		printf("%s %s: error[%d] set_app_config failed\n",
+			__func__, ifname, status);
+
+	if (last != protocol->Advertise && protocol->Advertise) {
+		tlv_enabletx(ifname, agenttype, (OUI_CEE_DCBX << 8) |
+			     protocol->dcbx_st);
+	}
+
+	eflag = *(ibuf+DCB_PORT_OFF+plen+CFG_ENABLE);
+	wflag = *(ibuf+DCB_PORT_OFF+plen+CFG_WILLING);
+
+	for (i = 0; i < DCB_MAX_APPTLV; i++) {
+		status = get_app(ifname, (u32)i, &app_data);
+		if (status != cmd_success)
+			continue;
+
+		protocol = &app_data.protocol;
+
+		if (eflag == '0' || eflag == '1')
+			protocol->Enable = eflag & 0x01;
+
+		if (wflag == '0' || wflag == '1')
+			protocol->Willing = wflag & 0x01;
+
+		status = put_app(ifname, i, &app_data);
+		if (status != cmd_success)
+			printf("%s %s: error[%d] set_app_config failed\n",
+				__func__, ifname, status);
+	}
+
+	somethingChangedLocal(ifname, agenttype);
+}
+
 static void set_protocol_data(feature_protocol_attribs *protocol, char *ifname,
-			      char *ibuf, int plen)
+			      char *ibuf, int plen, int agenttype)
 {
 	u8 flag;
 	int last;
@@ -389,11 +458,9 @@ static void set_protocol_data(feature_protocol_attribs *protocol, char *ifname,
 		last = protocol->Advertise;
 		protocol->Advertise = flag & 0x01;
 		if (last != protocol->Advertise && protocol->Advertise) {
-			if (init_cfg()) {
-				tlv_enabletx(ifname, (OUI_CEE_DCBX << 8) |
-					     protocol->dcbx_st);
-				somethingChangedLocal(ifname);
-			}
+			tlv_enabletx(ifname, agenttype, (OUI_CEE_DCBX << 8) |
+				     protocol->dcbx_st);
+			somethingChangedLocal(ifname, agenttype);
 		}
 	}
 
@@ -403,10 +470,10 @@ static void set_protocol_data(feature_protocol_attribs *protocol, char *ifname,
 }
 
 /* rbuf points to correct destination location */
-static dcb_result get_cmd_protocol_data(feature_protocol_attribs *protocol,
+static cmd_status get_cmd_protocol_data(feature_protocol_attribs *protocol,
 	u8 cmd, char *rbuf)
 {
-	dcb_result rval = dcb_success;
+	cmd_status rval = cmd_success;
 
 	switch(cmd) {
 	case CMD_GET_CONFIG:
@@ -433,20 +500,18 @@ static dcb_result get_cmd_protocol_data(feature_protocol_attribs *protocol,
 				protocol->Error,
 				protocol->dcbx_st);
 		else
-			rval = dcb_peer_not_present;
+			rval = cmd_peer_not_present;
 		break;
 	}
 	return rval;
 }
 
-static int  handle_dcbx_cmd(u8 cmd, u8 feature, u8 subtype,
-				     char *ibuf, int ilen,
-				     char *rbuf)
+static int handle_dcbx_cmd(u8 cmd, u8 feature, char *ibuf, int ilen, char *rbuf)
 {
-	int status = dcb_success;
+	int status = cmd_success;
 
 	if (ilen < DCBX_CFG_OFF)
-		return dcb_invalid_cmd;
+		return cmd_invalid;
 
 	/* append standard dcb command response content */
 	sprintf(rbuf , "%*.*s", DCBX_CFG_OFF, DCBX_CFG_OFF, ibuf);
@@ -458,7 +523,7 @@ static int  handle_dcbx_cmd(u8 cmd, u8 feature, u8 subtype,
 		} else if (cmd != CMD_GET_PEER) {
 			status = get_dcbx_config(cmd, rbuf+strlen(rbuf));
 		} else {
-			status = dcb_invalid_cmd;
+			status = cmd_invalid;
 		}
 		break;
 	default:
@@ -469,12 +534,12 @@ static int  handle_dcbx_cmd(u8 cmd, u8 feature, u8 subtype,
 }
 
 int dcbx_clif_cmd(void *data,
-		  struct sockaddr_un *from,
-		  socklen_t fromlen,
+		  UNUSED struct sockaddr_un *from,
+		  UNUSED socklen_t fromlen,
 		  char *ibuf, int ilen,
-		  char *rbuf)
+		  char *rbuf, int rlen)
 {
-	u8 status=dcb_success;
+	u8 status = cmd_success;
 	u8 cmd;
 	u8 feature;
 	u8 subtype;
@@ -484,43 +549,55 @@ int dcbx_clif_cmd(void *data,
 	pfc_attribs pfc_data;
 	app_attribs app_data;
 	llink_attribs llink_data;
+	struct port *port;
+	struct dcbx_tlvs *dcbx;
 
 	data = (struct clif_data *) data;
 
 	if (hexstr2bin(ibuf+DCB_CMD_OFF, &cmd, sizeof(cmd)) ||
 		hexstr2bin(ibuf+DCB_FEATURE_OFF, &feature, sizeof(feature)))
-		return dcb_invalid_cmd;
+		return cmd_invalid;
 
 	if (feature == FEATURE_DCBX)
-		return handle_dcbx_cmd(cmd, feature, subtype, ibuf, ilen, rbuf);
+		return handle_dcbx_cmd(cmd, feature, ibuf, ilen, rbuf);
 
 	if (hexstr2bin(ibuf+DCB_SUBTYPE_OFF, &subtype, sizeof(subtype)) ||
 		hexstr2bin(ibuf+DCB_PORTLEN_OFF, &plen, sizeof(plen)))
-		return dcb_invalid_cmd;
+		return cmd_invalid;
 
 	if (ilen < DCB_PORT_OFF)
-		return dcb_invalid_cmd;
+		return cmd_invalid;
 	
-	if (ibuf[DCB_VER_OFF] != (CLIF_MSG_VERSION | 0x30)) {
-		printf("unsupported client interface message version %x\n",
-			ibuf[DCB_VER_OFF]);
-		return dcb_invalid_cmd;
+	if (ibuf[DCB_VER_OFF] < (CLIF_DCBMSG_VERSION | 0x30)) {
+		printf("unsupported client interface message version %x %x\n",
+			ibuf[DCB_VER_OFF], CLIF_DCBMSG_VERSION | 0x30);
+		return cmd_ctrl_vers_not_compatible;
 	}
 
 	if (ilen < DCB_PORT_OFF+plen) {
 		printf("command too short\n");
-		return dcb_invalid_cmd;
+		return cmd_invalid;
 	}
 
 	/* append standard dcb command response content */
-	sprintf(rbuf , "%*.*s", DCB_PORT_OFF+plen, DCB_PORT_OFF+plen, ibuf);
+	snprintf(rbuf , rlen, "%*.*s",
+		 DCB_PORT_OFF+plen, DCB_PORT_OFF+plen, ibuf);
 
 	memcpy(port_id, ibuf+DCB_PORT_OFF, plen);
 	port_id[plen] = '\0';
 
-	/* check that interface is present and in DCB state */
-	if (feature != FEATURE_DCB && check_port_dcb_mode(port_id) == false)
-		return dcb_device_not_found;
+	/* Confirm port is a lldpad managed port */
+	port = port_find_by_name(port_id);
+	if (!port)
+		return cmd_device_not_found;
+
+	dcbx = dcbx_data(port->ifname);
+	if (!dcbx)
+		return cmd_device_not_found;
+
+	/* OPER and PEER cmd not applicable while in IEEE-DCBX modes */
+	if (dcbx->active == 0 && (cmd == CMD_GET_PEER || cmd == CMD_GET_OPER))
+		return cmd_not_applicable;
 
 	switch(feature) {
 	case FEATURE_DCB:
@@ -529,21 +606,17 @@ int dcbx_clif_cmd(void *data,
 		else if (cmd == CMD_GET_CONFIG)
 			status = get_dcb_state(port_id, rbuf+strlen(rbuf));
 		else
-			status = dcb_invalid_cmd;
+			status = cmd_invalid;
 		break;
 
 	case FEATURE_PG:
 		if (cmd == CMD_GET_PEER) {
 			status = get_peer_pg(port_id, &pg_data);
 		} else {
-			if (init_cfg()) {
-				status = get_pg(port_id, &pg_data);
-			} else {
-				status = dcb_failed;
-			}
+			status = get_pg(port_id, &pg_data);
 		}
 
-		if (status != dcb_success) {
+		if (status != cmd_success) {
 			printf("error[%d] getting PG data for %s\n",
 				status, port_id);
 			return status;
@@ -552,17 +625,17 @@ int dcbx_clif_cmd(void *data,
 		if (cmd == CMD_SET_CONFIG) {
 			if (ilen < (DCB_PORT_OFF + plen + CFG_LEN)) {
 				printf("set command too short\n");
-				status = dcb_invalid_cmd;
+				status = cmd_invalid;
 			} else {
 				set_protocol_data(&pg_data.protocol, port_id,
-						  ibuf, plen);
+						  ibuf, plen, NEAREST_BRIDGE);
 				status = set_pg_config(&pg_data, port_id, ibuf,
 					ilen);
 			}
 		} else {
 			status = get_cmd_protocol_data(&pg_data.protocol, cmd,
 					rbuf+strlen(rbuf));
-			if (status == dcb_success)
+			if (status == cmd_success)
 				status = get_pg_data(&pg_data, cmd, port_id,
 					pg_data.protocol.dcbx_st,
 					rbuf+strlen(rbuf));
@@ -573,14 +646,10 @@ int dcbx_clif_cmd(void *data,
 		if (cmd == CMD_GET_PEER) {
 			status = get_peer_pfc(port_id, &pfc_data);
 		} else {
-			if (init_cfg()) {
-				status = get_pfc(port_id, &pfc_data);
-			} else {
-				status = dcb_failed;
-			}
+			status = get_pfc(port_id, &pfc_data);
 		}
 
-		if (status != dcb_success) {
+		if (status != cmd_success) {
 			printf("error[%d] getting PFC data for %s\n",
 					status, port_id);
 			return status;
@@ -589,17 +658,17 @@ int dcbx_clif_cmd(void *data,
 		if (cmd == CMD_SET_CONFIG) {
 			if (ilen < (DCB_PORT_OFF + plen + CFG_LEN)) {
 				printf("set command too short\n");
-				status = dcb_failed;
+				status = cmd_failed;
 			} else {
 				set_protocol_data(&pfc_data.protocol, port_id,
-						  ibuf, plen);
+						  ibuf, plen, NEAREST_BRIDGE);
 				status = set_pfc_config(&pfc_data, port_id,
 					ibuf, ilen);
 			}
 		} else {
 			status = get_cmd_protocol_data(&pfc_data.protocol,
 				cmd, rbuf+strlen(rbuf));
-			if (status == dcb_success)
+			if (status == cmd_success)
 				status = get_pfc_data(&pfc_data, cmd,
 					port_id, pfc_data.protocol.dcbx_st,
 					rbuf+strlen(rbuf));
@@ -611,15 +680,10 @@ int dcbx_clif_cmd(void *data,
 		if (cmd == CMD_GET_PEER) {
 			status = get_peer_app(port_id, (u32)subtype, &app_data);
 		} else {
-			if (init_cfg()) {
-				status = get_app(port_id, (u32)subtype,
-					&app_data);
-			} else {
-				status = dcb_failed;
-			}
+			status = get_app(port_id, (u32)subtype, &app_data);
 		}
 
-		if (status != dcb_success) {
+		if (status != cmd_success) {
 			printf("error[%d] getting APP data for %s\n", status,
 				port_id);
 			return status;
@@ -628,20 +692,17 @@ int dcbx_clif_cmd(void *data,
 		if (cmd == CMD_SET_CONFIG) {
 			if (ilen < (DCB_PORT_OFF + plen + CFG_LEN)) {
 				printf("set command too short\n");
-				status = dcb_failed;
+				status = cmd_failed;
 			} else {
-				set_protocol_data(&app_data.protocol, port_id,
-						  ibuf, plen);
-				status = set_app_config(&app_data, port_id,
-					(u32)subtype, ibuf, ilen);
+				set_app_protocol_data(port_id, ibuf, plen, ilen,
+						      subtype, NEAREST_BRIDGE);
 			}
 		} else {
 			status = get_cmd_protocol_data(&app_data.protocol, cmd,
 				rbuf+strlen(rbuf));
-			if (status == dcb_success)
+			if (status == cmd_success)
 				status = get_app_data(&app_data, cmd, port_id,
-					subtype, app_data.protocol.dcbx_st,
-					rbuf+strlen(rbuf));
+					subtype, rbuf + strlen(rbuf));
 		}
 		break;
 
@@ -650,15 +711,10 @@ int dcbx_clif_cmd(void *data,
 			status = get_peer_llink(port_id, (u32)subtype,
 				&llink_data);
 		} else {
-			if (init_cfg()) {
-				status = get_llink(port_id, (u32)subtype,
-					&llink_data);
-			} else {
-				status = dcb_failed;
-			}
+			status = get_llink(port_id, (u32)subtype, &llink_data);
 		}
 
-		if (status != dcb_success) {
+		if (status != cmd_success) {
 			printf("error[%d] getting APP data for %s\n", status,
 				port_id);
 			return status;
@@ -667,34 +723,28 @@ int dcbx_clif_cmd(void *data,
 		if (cmd == CMD_SET_CONFIG) {
 			if (ilen < (DCB_PORT_OFF + plen + CFG_LEN)) {
 				printf("set command too short\n");
-				status = dcb_failed;
+				status = cmd_failed;
 			} else {
 				set_protocol_data(&llink_data.protocol, port_id,
-						  ibuf, plen);
+						  ibuf, plen, NEAREST_BRIDGE);
 				status = set_llink_config(&llink_data, port_id,
 					(u32)subtype, ibuf, ilen);
 			}
 		} else {
 			status = get_cmd_protocol_data(&llink_data.protocol,
 				cmd, rbuf+strlen(rbuf));
-			if (status == dcb_success)
+			if (status == cmd_success)
 				status = get_llink_data(&llink_data, cmd,
-					port_id, subtype,
-					llink_data.protocol.dcbx_st,
-					rbuf+strlen(rbuf));
+					port_id, subtype, rbuf + strlen(rbuf));
 		}
 		break;
 
 	case FEATURE_PG_DESC:
 		if (cmd == CMD_GET_CONFIG) {
-			if (init_cfg()) {
-				status = get_bwg_desc(port_id, ibuf, ilen,
-					rbuf+strlen(rbuf));
-			} else {
-				status = dcb_failed;
-			}
+			status = get_bwg_desc(port_id, ibuf, ilen,
+					      rbuf+strlen(rbuf));
 
-			if (status != dcb_success) {
+			if (status != cmd_success) {
 				printf("error[%d] getting BWG desc for %s\n",
 					status, port_id);
 				return status;
@@ -713,7 +763,7 @@ int dcbx_clif_cmd(void *data,
 
 
 /* rbuf points to correct destination location */
-static dcb_result get_pg_data(pg_attribs *pg_data, int cmd, char *port_id,
+static cmd_status get_pg_data(pg_attribs *pg_data, int cmd, char *port_id,
 	int dcbx_st, char *rbuf)
 {
 	int i;
@@ -726,7 +776,7 @@ static dcb_result get_pg_data(pg_attribs *pg_data, int cmd, char *port_id,
 		break;
 	case CMD_GET_OPER:
 		status = get_oper_pg(port_id, pg_data);
-		if (status != dcb_success) {
+		if (status != cmd_success) {
 			printf("error[%d] getting oper PG data for %s\n",
 				status, port_id);
 			return status;
@@ -735,7 +785,7 @@ static dcb_result get_pg_data(pg_attribs *pg_data, int cmd, char *port_id,
 	}
 
 	for (i = 0; i < MAX_USER_PRIORITIES; i++)
-		sprintf(rbuf+PG_UP2TC(i), "%1x", pg_data->tx.up[i].tcmap);
+		sprintf(rbuf+PG_UP2TC(i), "%1x", pg_data->tx.up[i].pgid);
 	for (i = 0; i < MAX_BANDWIDTH_GROUPS; i++)
 		sprintf(rbuf+PG_PG_PCNT(i), "%02x", pg_data->tx.pg_percent[i]);
 	for (i = 0; i < MAX_USER_PRIORITIES; i++) {
@@ -773,7 +823,7 @@ static dcb_result get_pg_data(pg_attribs *pg_data, int cmd, char *port_id,
 	else
 		sprintf(rbuf+PG_UP_NUM_TC, "%1x", pg_data->num_tcs);
 
-	return dcb_success;
+	return cmd_success;
 }
 
 /* rbuf points to correct destination location */
@@ -788,7 +838,7 @@ static int get_pfc_data(pfc_attribs *pfc_data, int cmd, char *port_id,
 		break;
 	case CMD_GET_OPER:
 		status = get_oper_pfc(port_id, pfc_data);
-		if (status != dcb_success) {
+		if (status != cmd_success) {
 			printf("error[%d] getting oper PFC data for %s\n",
 				status, port_id);
 			return status;
@@ -796,7 +846,7 @@ static int get_pfc_data(pfc_attribs *pfc_data, int cmd, char *port_id,
 		break;
 	case CMD_GET_PEER:
 		status = get_peer_pfc(port_id, pfc_data);
-		if (status != dcb_success) {
+		if (status != cmd_success) {
 			printf("error[%d] getting peer PFC data for %s\n",
 				status, port_id);
 			return status;
@@ -814,12 +864,12 @@ static int get_pfc_data(pfc_attribs *pfc_data, int cmd, char *port_id,
 	else
 		sprintf(rbuf+PFC_NUM_TC, "%1x", pfc_data->num_tcs);
 
-	return dcb_success;
+	return cmd_success;
 }
 
 /* rbuf points to correct destination location */
 static int get_app_data(app_attribs *app_data, int cmd, char *port_id,
-		u32 subtype, int dcbx_st, char *rbuf)
+		u32 subtype, char *rbuf)
 {
 	int status;
 	unsigned int i;
@@ -829,7 +879,7 @@ static int get_app_data(app_attribs *app_data, int cmd, char *port_id,
 		break;
 	case CMD_GET_OPER:
 		status = get_oper_app(port_id, subtype, app_data);
-		if (status != dcb_success) {
+		if (status != cmd_success) {
 			printf("error[%d] getting oper App data for %s\n",
 				status, port_id);
 			return status;
@@ -837,7 +887,7 @@ static int get_app_data(app_attribs *app_data, int cmd, char *port_id,
 		break;
 	case CMD_GET_PEER:
 		status = get_peer_app(port_id, subtype, app_data);
-		if (status != dcb_success) {
+		if (status != cmd_success) {
 			printf("error[%d] getting peer app data for %s\n",
 				status, port_id);
 			return status;
@@ -850,12 +900,12 @@ static int get_app_data(app_attribs *app_data, int cmd, char *port_id,
 	for (i=0; i<app_data->Length; i++)
 		sprintf(rbuf+APP_DATA+2*i, "%02x", *(app_data->AppData+i));
 
-	return dcb_success;
+	return cmd_success;
 }
 
 /* rbuf points to correct destination location */
 static int get_llink_data(llink_attribs *llink_data, int cmd, char *port_id,
-		u32 subtype, int dcbx_st, char *rbuf)
+		u32 subtype, char *rbuf)
 {
 	int status;
 
@@ -864,7 +914,7 @@ static int get_llink_data(llink_attribs *llink_data, int cmd, char *port_id,
 		break;
 	case CMD_GET_OPER:
 		status = get_oper_llink(port_id, subtype, llink_data);
-		if (status != dcb_success) {
+		if (status != cmd_success) {
 			printf("error[%d] getting oper logical link data "
 				"for %s\n", status, port_id);
 			return status;
@@ -872,7 +922,7 @@ static int get_llink_data(llink_attribs *llink_data, int cmd, char *port_id,
 		break;
 	case CMD_GET_PEER:
 		status = get_peer_llink(port_id, subtype, llink_data);
-		if (status != dcb_success) {
+		if (status != cmd_success) {
 			printf("error[%d] getting peer logical link data "
 				"for %s\n", status, port_id);
 			return status;
@@ -882,17 +932,18 @@ static int get_llink_data(llink_attribs *llink_data, int cmd, char *port_id,
 	
 	sprintf(rbuf+LLINK_STATUS, "%1x", llink_data->llink.llink_status);
 
-	return dcb_success;
+	return cmd_success;
 }
 
 
 static int set_pg_config(pg_attribs *pg_data, char *port_id, char *ibuf,
 	int ilen)
 {
+	pfc_attribs pfc_data;
 	full_dcb_attrib_ptrs dcb_data;
 	u8 flag;
-	dcb_result status = dcb_success;
-	int i;
+	cmd_status status = cmd_success;
+	int i, is_pfc;
 	int plen;
 	int off;
 	bool used[MAX_BANDWIDTH_GROUPS];
@@ -906,8 +957,12 @@ static int set_pg_config(pg_attribs *pg_data, char *port_id, char *ibuf,
 			flag = *(ibuf+off+PG_UP2TC(i));
 			if (flag == CLIF_NOT_SUPPLIED)
 				continue;
-			pg_data->tx.up[i].tcmap = flag & 0x07;
-			pg_data->rx.up[i].tcmap = flag & 0x07;
+
+			if ((flag & 0x07) >= pg_data->num_tcs)
+				return cmd_bad_params;
+
+			pg_data->tx.up[i].pgid = flag & 0x07;
+			pg_data->rx.up[i].pgid = flag & 0x07;
 		}
 		memset(used, false, sizeof(used));
 		for (i = 0; i < MAX_BANDWIDTH_GROUPS; i++) {
@@ -915,7 +970,7 @@ static int set_pg_config(pg_attribs *pg_data, char *port_id, char *ibuf,
 				continue;
 			if (hexstr2bin(ibuf+off+PG_PG_PCNT(i),
 				&flag, sizeof(flag)))
-				return dcb_bad_params;
+				return cmd_bad_params;
 			pg_data->tx.pg_percent[i] = flag;
 			pg_data->rx.pg_percent[i] = flag;
 		}
@@ -950,7 +1005,7 @@ static int set_pg_config(pg_attribs *pg_data, char *port_id, char *ibuf,
 			uppcts_changed = true;
 			if (hexstr2bin(ibuf+off+PG_UP_PCNT(i),
 				&flag, sizeof(flag)))
-				return dcb_bad_params;
+				return cmd_bad_params;
 			pg_data->tx.up[i].percent_of_pg_cap = flag;
 			pg_data->rx.up[i].percent_of_pg_cap = flag;
 		}
@@ -997,7 +1052,7 @@ static int set_pg_config(pg_attribs *pg_data, char *port_id, char *ibuf,
 		}
 	} else if (ilen != off) {
 		printf("error - setcommand has invalid argument length\n");
-		return dcb_bad_params;
+		return cmd_bad_params;
 	}
 
 	memset((void *)&dcb_data, 0, sizeof(dcb_data));
@@ -1008,22 +1063,23 @@ static int set_pg_config(pg_attribs *pg_data, char *port_id, char *ibuf,
 	 * user priority percentages, then compute new percentages
 	 * and try one more time.
 	*/
-	if (status == dcb_bad_params && !uppcts_changed) {
+	if (status == cmd_bad_params && !uppcts_changed) {
 		rebalance_uppcts(pg_data);
 		status = dcb_check_config(&dcb_data);
 	}
 
-	if (status != dcb_success) {
+	if (status != cmd_success) {
 		printf("invalid DCB settings\n");
 		return status;
 	}
 
-	if (init_cfg()) {
-		status = put_pg(port_id, pg_data);
-	} else {
-		status = dcb_failed;
-	}
-	if (status != dcb_success)
+	is_pfc = get_pfc(port_id, &pfc_data);
+	if (is_pfc == cmd_success)
+		status = put_pg(port_id, pg_data, &pfc_data);
+	else
+		status = put_pg(port_id, pg_data, NULL);
+
+	if (status != cmd_success)
 		printf("error[%d] setting PG data for %s\n", status, port_id);
 	
 
@@ -1034,7 +1090,7 @@ static int set_pfc_config(pfc_attribs *pfc_data, char *port_id, char *ibuf,
 	int ilen)
 {
 	u8 flag;
-	dcb_result status = dcb_success;
+	cmd_status status = cmd_success;
 	int i;
 	int off;
 	int plen;
@@ -1055,15 +1111,11 @@ static int set_pfc_config(pfc_attribs *pfc_data, char *port_id, char *ibuf,
 	} else if (ilen != off) {
 		/* at least needs to include the protocol settings */
 		printf("error - setcommand has invalid argument length\n");
-		return dcb_failed;
+		return cmd_failed;
 	}
 
-	if (init_cfg()) {
-		status = put_pfc(port_id, pfc_data);
-	} else {
-		status = dcb_failed;
-	}
-	if (status != dcb_success)
+	status = put_pfc(port_id, pfc_data);
+	if (status != cmd_success)
 		printf("error[%d] setting PFC data for %s\n", status, port_id);
 
 	return status;
@@ -1072,7 +1124,7 @@ static int set_pfc_config(pfc_attribs *pfc_data, char *port_id, char *ibuf,
 static int set_app_config(app_attribs *app_data, char *port_id, u32 subtype,
 		char *ibuf, int ilen)
 {
-	dcb_result status = dcb_success;
+	cmd_status status = cmd_success;
 	int off;
 	int plen;
 	u8 applen = 0;
@@ -1085,19 +1137,16 @@ static int set_app_config(app_attribs *app_data, char *port_id, u32 subtype,
 	if (ilen == (off + APP_DATA + applen)) {
 		app_data->Length = applen/2;
 		if (hexstr2bin(ibuf+off+APP_DATA, app_data->AppData, applen/2))
-			return dcb_bad_params;
+			return cmd_bad_params;
 	} else if (ilen != off) {
 		/* at least needs to include the protocol settings */
 		printf("error - setcommand has invalid argument length\n");
-		return dcb_failed;
+		return cmd_failed;
 	}
 
-	if (init_cfg()) {
-		status = put_app(port_id, subtype, app_data);
-	} else {
-		status = dcb_failed;
-	}
-	if (status != dcb_success)
+	status = put_app(port_id, subtype, app_data);
+
+	if (status != cmd_success)
 		printf("error[%d] setting APP data for %s\n", status, port_id);
 
 	return status;
@@ -1106,7 +1155,7 @@ static int set_app_config(app_attribs *app_data, char *port_id, u32 subtype,
 static int set_llink_config(llink_attribs *llink_data, char *port_id,
 		u32 subtype, char *ibuf, int ilen)
 {
-	dcb_result status = dcb_success;
+	cmd_status status = cmd_success;
 	int off;
 	int value;
 	
@@ -1115,20 +1164,16 @@ static int set_llink_config(llink_attribs *llink_data, char *port_id,
 	if (ilen != off + LLINK_DLEN) {
 		/* at least needs to include the protocol settings */
 		printf("error - setcommand has invalid argument length\n");
-		return dcb_failed;
+		return cmd_failed;
 	}
 
 	value = *(ibuf+off+LLINK_STATUS);
 	if (value != CLIF_NOT_SUPPLIED)
 		llink_data->llink.llink_status = value & 0x0f;
 
-	if (init_cfg()) {
-		status = put_llink(port_id, subtype, llink_data);
-	} else {
-		status = dcb_failed;
-	}
+	status = put_llink(port_id, subtype, llink_data);
 
-	if (status != dcb_success)
+	if (status != cmd_success)
 		printf("error[%d] setting logical link data for %s\n",
 			status, port_id);
 
