@@ -1,7 +1,7 @@
 /*******************************************************************************
 
   LLDP Agent Daemon (LLDPAD) Software
-  Copyright(c) 2007-2010 Intel Corporation.
+  Copyright(c) 2007-2012 Intel Corporation.
 
   This program is free software; you can redistribute it and/or modify it
   under the terms and conditions of the GNU General Public License,
@@ -20,13 +20,14 @@
   the file called "COPYING".
 
   Contact Information:
-  e1000-eedc Mailing List <e1000-eedc@lists.sourceforge.net>
-  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497
+  open-lldp Mailing List <lldp-devel@open-lldp.org>
 
 *******************************************************************************/
 
+#include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <errno.h>
 #include <ifaddrs.h>
 #include <netdb.h>
 #include <unistd.h>
@@ -34,20 +35,120 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <net/if_arp.h>
+#include <netlink/msg.h>
 #include <arpa/inet.h>
-#include <linux/if.h>
-#include <linux/if_bonding.h>
-#include <linux/if_bridge.h>
-#include <linux/if_vlan.h>
 #include <linux/wireless.h>
 #include <linux/sockios.h>
-#include <linux/ethtool.h>
 #include <dirent.h>
+#include "linux/if_bonding.h"
+#include "linux/if_bridge.h"
+#include "linux/ethtool.h"
+#include "linux/rtnetlink.h"
+#include "linux/if_vlan.h"
+#include "linux/if.h"
 #include "lldp.h"
 #include "lldp_util.h"
 #include "messages.h"
-#include "drv_cfg.h"
+#include "lldp_dcbx_nl.h"
 
+static int hex2num(char c)
+{
+	if (c >= '0' && c <= '9')
+		return c - '0';
+	if (c >= 'a' && c <= 'f')
+		return c - 'a' + 10;
+	if (c >= 'A' && c <= 'F')
+		return c - 'A' + 10;
+	return -1;
+}
+
+static int hex2byte(const char *hex)
+{
+	int a, b;
+	a = hex2num(*hex++);
+	if (a < 0)
+		return -1;
+	b = hex2num(*hex++);
+	if (b < 0)
+		return -1;
+	return (a << 4) | b;
+}
+
+/**
+ * bin2hexstr - Convert binary data to ASCII string
+ * @hex: ASCII hex string (e.g., "01ab")
+ * @buf: Buffer for the binary data
+ * @len: Length of the text to convert in bytes (of buf); hex will be double
+ * this size
+ * Returns: 0 on success, -1 on failure (invalid hex string)
+ */
+#define BYTE2CHAR(b)	(((b) > 9) ? ((b) - 0xa + 'A') : ((b) + '0'))
+int bin2hexstr(const u8 *hex, size_t hexlen, char *buf, size_t buflen)
+{
+	u8 b;
+	size_t i, j;
+
+	for (i = j = 0; (i < hexlen) && (j < buflen); i++, j +=2) {
+		b = (hex[i] & 0xf0) >> 4;
+		buf[j] = BYTE2CHAR(b);
+		b = hex[i] & 0x0f;
+		buf[j + 1] = BYTE2CHAR(b);
+	}
+	return 0;
+}
+
+/**
+ * hexstr2bin - Convert ASCII hex string into binary data
+ * @hex: ASCII hex string (e.g., "01ab")
+ * @buf: Buffer for the binary data
+ * @len: Length of the text to convert in bytes (of buf); hex will be double
+ * this size
+ * Returns: 0 on success, -1 on failure (invalid hex string)
+ */
+int hexstr2bin(const char *hex, u8 *buf, size_t len)
+{
+	size_t i;
+	int a;
+	const char *ipos = hex;
+	u8 *opos = buf;
+
+	for (i = 0; i < len; i++) {
+		a = hex2byte(ipos);
+		if (a < 0)
+			return -1;
+		*opos++ = a;
+		ipos += 2;
+	}
+	return 0;
+}
+
+char *print_mac(char *mac, char *buf)
+{
+	sprintf(buf, "%02x:%02x:%02x:%02x:%02x:%02x",
+		(unsigned char)*(mac + 0),
+		(unsigned char)*(mac + 1),
+		(unsigned char)*(mac + 2),
+		(unsigned char)*(mac + 3),
+		(unsigned char)*(mac + 4),
+		(unsigned char)*(mac + 5));
+	return buf;
+}
+
+static int get_ioctl_socket(void)
+{
+	static int ioctl_socket = -1;
+
+	if (ioctl_socket >= 0)
+		return ioctl_socket;
+	ioctl_socket = socket(PF_INET, SOCK_DGRAM, 0);
+	if (ioctl_socket < 0) {
+		int err = errno;
+
+		perror("socket create failed\n");
+		errno = err;
+	}
+	return ioctl_socket;
+}
 
 int is_valid_lldp_device(const char *device_name)
 {
@@ -57,6 +158,8 @@ int is_valid_lldp_device(const char *device_name)
 		return 0;
 	if (is_bridge(device_name))
 		return 0;
+	if (is_macvtap(device_name))
+		return 0;
 	return 1;
 }
 
@@ -68,22 +171,19 @@ int is_valid_lldp_device(const char *device_name)
  */
 int is_bond(const char *ifname)
 {
-	int fd = -1;
+	int fd;
 	int rc = 0;
 	struct ifreq ifr;
 	ifbond ifb;
 
-	fd = socket(PF_INET, SOCK_DGRAM, 0);
-	if (fd > 0) {
+	fd = get_ioctl_socket();
+	if (fd >= 0) {
 		memset(&ifr, 0, sizeof(ifr));
 		strcpy(ifr.ifr_name, ifname);
 		memset(&ifb, 0, sizeof(ifb));
 		ifr.ifr_data = (caddr_t)&ifb;
 		if (ioctl(fd, SIOCBONDINFOQUERY, &ifr) == 0)
 			rc = 1;
-		close(fd);
-	} else {
-		perror("is_bond() socket create failed");
 	}
 	return rc;
 }
@@ -131,12 +231,9 @@ int	get_src_mac_from_bond(struct port *bond_port, char *ifname, u8 *addr)
 	int found = 0;
 	int i;
 
-	fd = socket(PF_INET, SOCK_DGRAM, 0);
-
-	if (fd <= 0) {
-		perror("get_src_mac_from_bond failed");
+	fd = get_ioctl_socket();
+	if (fd < 0)
 		return 0;
-	}
 
 	memset(bond_mac, 0, sizeof(bond_mac));
 	memset(&ifr, 0, sizeof(ifr));
@@ -172,10 +269,8 @@ int	get_src_mac_from_bond(struct port *bond_port, char *ifname, u8 *addr)
 	}
 
 	/* current port is not a slave of the bond */
-	if (!found) {
-		close(fd);
+	if (!found)
 		return 0;
-	}
 
 	/* Get slave port's current perm MAC address
 	 * This will be the default return value
@@ -187,7 +282,6 @@ int	get_src_mac_from_bond(struct port *bond_port, char *ifname, u8 *addr)
 	}
 	else {
 		perror("error getting slave MAC address");
-		close(fd);
 		return 0;
 	}
 
@@ -211,8 +305,6 @@ int	get_src_mac_from_bond(struct port *bond_port, char *ifname, u8 *addr)
 		break;
 	}
 
-	close(fd);
-
 	return 1;
 }
 
@@ -242,40 +334,34 @@ int read_bool(const char *path)
 
 int get_ifflags(const char *ifname)
 {
-	int fd = -1;
+	int fd;
 	int flags = 0;
 	struct ifreq ifr;
 
 	/* use ioctl */	
-	fd = socket(AF_LOCAL, SOCK_STREAM, 0);
-	if (fd > 0) {
+	fd = get_ioctl_socket();
+	if (fd >= 0) {
 		memset(&ifr, 0, sizeof(ifr));
 		strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));
-		if (ioctl(fd, SIOCGIFFLAGS, &ifr) == 0) {
+		if (ioctl(fd, SIOCGIFFLAGS, &ifr) == 0)
 			flags = ifr.ifr_flags;
-		} else {
-		}
-		close(fd);
 	}
 	return flags;
 }
 
 int get_ifpflags(const char *ifname)
 {
-	int fd = -1;
+	int fd;
 	int flags = 0;
 	struct ifreq ifr;
 
 	/* use ioctl */	
-	fd = socket(AF_LOCAL, SOCK_STREAM, 0);
-	if (fd > 0) {
+	fd = get_ioctl_socket();
+	if (fd >= 0) {
 		memset(&ifr, 0, sizeof(ifr));
 		strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));
-		if (ioctl(fd, SIOCGIFPFLAGS, &ifr) == 0) {
+		if (ioctl(fd, SIOCGIFPFLAGS, &ifr) == 0)
 			flags = ifr.ifr_flags;
-		} else {
-		}
-		close(fd);
 	}
 	return flags;
 }
@@ -288,15 +374,6 @@ int get_iftype(const char *ifname)
 	return read_int(path);
 }
 
-int get_ifindex(const char *ifname)
-{
-	char path[256];
-
-	snprintf(path, sizeof(path), "/sys/class/net/%s/ifindex", ifname);
-	return read_int(path);
-
-}
-
 int get_iffeatures(const char *ifname)
 {
 	char path[256];
@@ -360,8 +437,8 @@ int is_slave(const char *ifmaster, const char *ifslave)
 	if (!is_mbond(ifmaster))
 		goto out_done;
 
-	fd = socket(PF_INET, SOCK_DGRAM, 0);
-	if (fd <= 0)
+	fd = get_ioctl_socket();
+	if (fd < 0)
 		goto out_done;
 
 	memset(&ifr, 0, sizeof(ifr));
@@ -369,7 +446,7 @@ int is_slave(const char *ifmaster, const char *ifslave)
 	strncpy(ifr.ifr_name, ifmaster, IFNAMSIZ);
 	ifr.ifr_data = (caddr_t)&ifb;
 	if (ioctl(fd, SIOCBONDINFOQUERY, &ifr))
-		goto out_close;
+		goto out_done;
 
 	for (i = 0; i < ifb.num_slaves; i++) {
 		memset(&ifs, 0, sizeof(ifs));
@@ -383,8 +460,6 @@ int is_slave(const char *ifmaster, const char *ifslave)
 		}
 	}
 	
-out_close:
-	close(fd);
 out_done:
 	return rc;
 }
@@ -395,13 +470,12 @@ int get_ifidx(const char *ifname)
 	int idx = 0;
 	struct ifreq ifreq;
 
-	fd = socket(AF_LOCAL, SOCK_DGRAM, 0);
-	if (fd > 0) {
+	fd = get_ioctl_socket();
+	if (fd >= 0) {
 		memset(&ifreq, 0, sizeof(ifreq));
 		strncpy(ifreq.ifr_name, ifname, IFNAMSIZ);
 		if (ioctl(fd, SIOCGIFINDEX, &ifreq) == 0)
 			idx = ifreq.ifr_ifindex;
-		close(fd);
 	}
 	return idx;
 }
@@ -410,8 +484,8 @@ int get_master(const char *ifname)
 {
 	int i;
 	int idx = 0;
-	int fd = -1;
-	int cnt = 0;
+	int fd;
+	int cnt;
 	struct ifreq *ifr = NULL;
 	struct ifconf ifc;
 	char ifcbuf[sizeof(struct ifreq) * 32];
@@ -420,8 +494,8 @@ int get_master(const char *ifname)
 	if (is_mbond(ifname))
 		return get_ifidx(ifname);
 
-	fd = socket(PF_INET, SOCK_DGRAM, 0);
-	if (fd > 0) {
+	fd = get_ioctl_socket();
+	if (fd >= 0) {
 		memset(&ifc, 0, sizeof(ifc));
 		memset(ifcbuf, 0, sizeof(ifcbuf));
 		ifc.ifc_buf = ifcbuf;
@@ -441,14 +515,13 @@ int get_master(const char *ifname)
 				break;
 			}
 		}
-		close(fd);
 	}
 	return idx;
 }
 
 int is_bridge(const char *ifname)
 {
-	int fd = -1;
+	int fd;
 	int rc = 0;
 	char path[256];
 	DIR *dirp;
@@ -464,8 +537,8 @@ int is_bridge(const char *ifname)
 		rc = 1;
 	} else { 
 		/* use ioctl */	
-		fd = socket(AF_LOCAL, SOCK_STREAM, 0);
-		if (fd > 0) {
+		fd = get_ioctl_socket();
+		if (fd >= 0) {
 			struct ifreq ifr;
 			struct __bridge_info bi;
 			unsigned long args[4] = { BRCTL_GET_BRIDGE_INFO, 
@@ -473,12 +546,8 @@ int is_bridge(const char *ifname)
 
 			ifr.ifr_data = (char *)args;
 			strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));
-			if (ioctl(fd, SIOCDEVPRIVATE, &ifr) == 0) {
+			if (ioctl(fd, SIOCDEVPRIVATE, &ifr) == 0)
 				rc = 1;
-			}
-			close(fd);
-		} else {
-			perror("is_bridge() open socket failed");
 		}
 	}
 	return rc;
@@ -486,20 +555,17 @@ int is_bridge(const char *ifname)
 
 int is_vlan(const char *ifname)
 {
-	int fd = -1;
+	int fd;
 	int rc = 0;
 	struct vlan_ioctl_args ifv;
 
-	fd = socket(PF_INET, SOCK_DGRAM, 0);
-	if (fd > 0) {
+	fd = get_ioctl_socket();
+	if (fd >= 0) {
 		memset(&ifv, 0, sizeof(ifv));
 		ifv.cmd = GET_VLAN_REALDEV_NAME_CMD;
 		strncpy(ifv.device1, ifname, sizeof(ifv.device1));
 		if (ioctl(fd, SIOCGIFVLAN, &ifv) == 0)
 			rc = 1;
-		close(fd);
-	} else {
-		perror("is_vlan() open socket error");
 	}
 	return rc;
 }
@@ -517,24 +583,108 @@ int is_vlan_capable(const char *ifname)
 
 int is_wlan(const char *ifname)
 {
-	int fd = -1;
+	int fd;
 	int rc = 0;
 	struct iwreq iwreq;
 
-	fd = socket(PF_INET, SOCK_DGRAM, 0);
-	if (fd > 0) {
+	fd = get_ioctl_socket();
+	if (fd >= 0) {
 		memset(&iwreq, 0, sizeof(iwreq));
 		strncpy(iwreq.ifr_name, ifname, sizeof(iwreq.ifr_name));
 		if (ioctl(fd, SIOCGIWNAME, &iwreq) == 0)
 			rc = 1;
-		close(fd);
-	} else {
-		perror("is_wlan() open socket error");
 	}
 	return rc;
 }
 
-int is_router(const char *ifname)
+#define NLMSG_SIZE 1024
+
+static struct nla_policy ifla_info_policy[IFLA_INFO_MAX + 1] =
+{
+  [IFLA_INFO_KIND]       = { .type = NLA_STRING},
+  [IFLA_INFO_DATA]       = { .type = NLA_NESTED },
+};
+
+int is_macvtap(const char *ifname)
+{
+	int ret, s;
+	struct nlmsghdr *nlh;
+	struct ifinfomsg *ifinfo;
+	struct nlattr *tb[IFLA_MAX+1],
+		      *tb2[IFLA_INFO_MAX+1];
+
+	s = socket(PF_NETLINK, SOCK_DGRAM, NETLINK_ROUTE);
+
+	if (s < 0) {
+		goto out;
+	}
+
+	nlh = malloc(NLMSG_SIZE);
+
+	if (!nlh) {
+		goto out;
+	}
+
+	memset(nlh, 0, NLMSG_SIZE);
+
+	nlh->nlmsg_len = NLMSG_LENGTH(sizeof(struct ifinfomsg));
+        nlh->nlmsg_type = RTM_GETLINK;
+        nlh->nlmsg_flags = NLM_F_REQUEST;
+
+	ifinfo = NLMSG_DATA(nlh);
+	ifinfo->ifi_family = AF_UNSPEC;
+	ifinfo->ifi_index = get_ifidx(ifname);
+
+	ret = send(s, nlh, nlh->nlmsg_len, 0);
+
+	if (ret < 0) {
+		goto out_free;
+	}
+
+	memset(nlh, 0, NLMSG_SIZE);
+
+	do {
+		ret = recv(s, (void *) nlh, NLMSG_SIZE, MSG_DONTWAIT);
+	} while ((ret < 0) && errno == EINTR);
+
+	if (nlmsg_parse(nlh, sizeof(struct ifinfomsg),
+			(struct nlattr **)&tb, IFLA_MAX, NULL)) {
+		goto out_free;
+	}
+
+	if (tb[IFLA_IFNAME]) {
+		ifname = (char *)RTA_DATA(tb[IFLA_IFNAME]);
+	} else {
+		ifinfo = (struct ifinfomsg *)NLMSG_DATA(nlh);
+	}
+
+	if (tb[IFLA_LINKINFO]) {
+		if (nla_parse_nested(tb2, IFLA_INFO_MAX, tb[IFLA_LINKINFO],
+				     ifla_info_policy)) {
+			goto out_free;
+		}
+
+		if (tb2[IFLA_INFO_KIND]) {
+			char *kind = (char*)(RTA_DATA(tb2[IFLA_INFO_KIND]));
+			if (!(strcmp("macvtap", kind) && strcmp("macvlan", kind))) {
+				free(nlh);
+				close(s);
+				return true;
+			}
+		}
+
+	} else {
+		goto out_free;
+	}
+
+out_free:
+	free(nlh);
+out:
+	close(s);
+	return false;
+}
+
+static int is_router(void)
 {
 	int rc = 0;
 	char path[256];
@@ -550,20 +700,17 @@ int is_router(const char *ifname)
 
 int is_active(const char *ifname)
 {
-	int fd = -1;
+	int fd;
 	int rc = 0;
 	struct ifreq ifr;
 
-	fd = socket(PF_INET, SOCK_DGRAM, 0);
-	if (fd > 0) {
+	fd = get_ioctl_socket();
+	if (fd >= 0) {
 		memset(&ifr, 0, sizeof(ifr));
 		strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));
 		if (ioctl(fd, SIOCGIFFLAGS, &ifr) == 0)
 			if (ifr.ifr_flags & IFF_UP)
 				rc = 1;
-		close(fd);
-	} else {
-		perror("is_active() open socket error");
 	}
 	return rc;
 }
@@ -571,12 +718,12 @@ int is_active(const char *ifname)
 int is_autoneg_supported(const char *ifname)
 {
 	int rc = 0;
-	int fd = -1;
+	int fd;
 	struct ifreq ifr;
 	struct ethtool_cmd cmd;
 	
-	fd = socket(AF_INET, SOCK_DGRAM, 0);
-	if (fd > 0) {
+	fd = get_ioctl_socket();
+	if (fd >= 0) {
 		memset(&ifr, 0, sizeof(ifr));
 		memset(&cmd, 0, sizeof(cmd));
 		cmd.cmd = ETHTOOL_GSET;
@@ -585,9 +732,6 @@ int is_autoneg_supported(const char *ifname)
 		if (ioctl(fd, SIOCETHTOOL, &ifr) == 0)
 			if (cmd.supported & SUPPORTED_Autoneg)
 				rc = 1;
-		close(fd);
-	} else {	
-		perror("is_autoneg_supported() open socket failed");
 	}
 	return rc;
 }
@@ -595,12 +739,12 @@ int is_autoneg_supported(const char *ifname)
 int is_autoneg_enabled(const char *ifname)
 {
 	int rc = 0;
-	int fd = -1;
+	int fd;
 	struct ifreq ifr;
 	struct ethtool_cmd cmd;
 	
-	fd = socket(AF_INET, SOCK_DGRAM, 0);
-	if (fd > 0) {
+	fd = get_ioctl_socket();
+	if (fd >= 0) {
 		memset(&ifr, 0, sizeof(ifr));
 		memset(&cmd, 0, sizeof(cmd));
 		cmd.cmd = ETHTOOL_GSET;
@@ -608,9 +752,6 @@ int is_autoneg_enabled(const char *ifname)
 		strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));
 		if (ioctl(fd, SIOCETHTOOL, &ifr) == 0)
 			rc = cmd.autoneg;
-		close(fd);
-	} else {	
-		perror("is_autoneg_enabled() open socket failed");
 	}
 	return rc;
 }
@@ -634,13 +775,13 @@ int is_autoneg_enabled(const char *ifname)
 #define MAUCAPADV_b1000baseTFD	(1 << 15) /* 1000BASE-T full duplex mode */
 int get_maucaps(const char *ifname)
 {
-	int fd = -1;
+	int fd;
 	u16 caps = MAUCAPADV_bOther;
 	struct ifreq ifr;
 	struct ethtool_cmd cmd;
 	
-	fd = socket(AF_INET, SOCK_DGRAM, 0);
-	if (fd > 0) {
+	fd = get_ioctl_socket();
+	if (fd >= 0) {
 		memset(&ifr, 0, sizeof(ifr));
 		memset(&cmd, 0, sizeof(cmd));
 		cmd.cmd = ETHTOOL_GSET;
@@ -666,54 +807,20 @@ int get_maucaps(const char *ifname)
 			if (cmd.advertising & (ADVERTISED_Asym_Pause | ADVERTISED_Pause))
 				caps |= MAUCAPADV_bFdxBPause;
 		}
-		close(fd);
-	} else {	
-		perror("is_autoneg() open socket failed");
 	}
 	return caps;
 }
 
-/* IETF RFC 2668 dot3MauType: http://www.rfc-editor.org/rfc/rfc2668.txt */
-#define DOT3MAUTYPE_AUI 1
-#define DOT3MAUTYPE_10Base5 2
-#define DOT3MAUTYPE_Foirl 3
-#define DOT3MAUTYPE_10Base2 4
-#define DOT3MAUTYPE_10BaseT 5
-#define DOT3MAUTYPE_10BaseFP 6
-#define DOT3MAUTYPE_10BaseFB 7
-#define DOT3MAUTYPE_10BaseFL 8
-#define DOT3MAUTYPE_10Broad36 9
-#define DOT3MAUTYPE_10BaseTHD 10
-#define DOT3MAUTYPE_10BaseTFD 11
-#define DOT3MAUTYPE_10BaseFLHD 12
-#define DOT3MAUTYPE_10BaseFLFD 13
-#define DOT3MAUTYPE_100BaseT4 14
-#define DOT3MAUTYPE_100BaseTXHD 15
-#define DOT3MAUTYPE_100BaseTXFD 16
-#define DOT3MAUTYPE_100BaseFXHD 17
-#define DOT3MAUTYPE_100BaseFXFD 18
-#define DOT3MAUTYPE_100BaseT2HD 19
-#define DOT3MAUTYPE_100BaseT2FD 20
-#define DOT3MAUTYPE_1000BaseXHD 21
-#define DOT3MAUTYPE_1000BaseXFD 22
-#define DOT3MAUTYPE_1000BaseLXHD 23
-#define DOT3MAUTYPE_1000BaseLXFD 24
-#define DOT3MAUTYPE_1000BaseSXHD 25
-#define DOT3MAUTYPE_1000BaseSXFD 26
-#define DOT3MAUTYPE_1000BaseCXHD 27
-#define DOT3MAUTYPE_1000BaseCXFD 28
-#define DOT3MAUTYPE_1000BaseTHD 29
-#define DOT3MAUTYPE_1000BaseTFD 30
 int get_mautype(const char *ifname)
 {
 	int rc = 0;
-	int fd = -1;
+	int fd;
 	struct ifreq ifr;
 	struct ethtool_cmd cmd;
 	u32 speed;
 
-	fd = socket(AF_INET, SOCK_DGRAM, 0);
-	if (fd > 0) {
+	fd = get_ioctl_socket();
+	if (fd >= 0) {
 		memset(&ifr, 0, sizeof(ifr));
 		memset(&cmd, 0, sizeof(cmd));
 		cmd.cmd = ETHTOOL_GSET;
@@ -732,29 +839,22 @@ int get_mautype(const char *ifname)
 			else if (speed == SPEED_1000)
 				rc = DOT3MAUTYPE_1000BaseTFD;
 		}
-		close(fd);
-	} else {	
-		perror("is_autoneg() open socket failed");
 	}
 	return rc;
 }
 
 int get_mtu(const char *ifname)
 {
-	int fd = -1;
+	int fd;
 	int rc = 0;
 	struct ifreq ifr;
 
-	fd = socket(AF_LOCAL, SOCK_STREAM, 0);
-	if (fd > 0) {
+	fd = get_ioctl_socket();
+	if (fd >= 0) {
 		memset(&ifr, 0, sizeof(ifr));
 		strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));
-		if (ioctl(fd, SIOCGIFMTU, &ifr) == 0) {
+		if (ioctl(fd, SIOCGIFMTU, &ifr) == 0)
 			rc = ifr.ifr_mtu;
-		}
-		close(fd);
-	} else {
-		perror("get_mtu() socket create failed");
 	}
 	return rc;
 }
@@ -776,22 +876,19 @@ int get_mfs(const char *ifname)
 
 int get_mac(const char *ifname, u8 mac[])
 {
-	int fd = -1;
+	int fd;
 	int rc = EINVAL;
 	struct ifreq ifr;
 
 	memset(mac, 0, 6);
-	fd = socket(AF_INET, SOCK_DGRAM, 0);
-	if (fd > 0) {
+	fd = get_ioctl_socket();
+	if (fd >= 0) {
 		ifr.ifr_addr.sa_family = AF_INET;
 		strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));
 		if (!ioctl(fd, SIOCGIFHWADDR, &ifr)) {
 			memcpy(mac, ifr.ifr_hwaddr.sa_data, 6);
 			rc = 0;
 		}
-		close(fd);
-	} else {
-		perror("get_mac() socket create failed");
 	}
 	return rc;
 }
@@ -821,7 +918,7 @@ u16 get_caps(const char *ifname)
 	if (is_bridge(ifname))
 		caps |= SYSCAP_BRIDGE;
 
-	if (is_router(ifname))
+	if (is_router())
 		caps |= SYSCAP_ROUTER;
 
 	if (is_wlan(ifname))
@@ -847,19 +944,18 @@ u16 get_caps(const char *ifname)
 
 int get_saddr(const char *ifname, struct sockaddr_in *saddr)
 {
-	int fd = -1;
+	int fd;
 	int rc = EIO;
 	struct ifreq ifr;
 
-	fd = socket(AF_INET, SOCK_DGRAM, IPPROTO_IP);
-	if (fd > 0) {
+	fd = get_ioctl_socket();
+	if (fd >= 0) {
 		ifr.ifr_addr.sa_family = AF_INET;
 		strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));
 		if (ioctl(fd, SIOCGIFADDR, &ifr) == 0) {
 			memcpy(saddr, &ifr.ifr_addr, sizeof(*saddr));
 			rc = 0;
 		}
-		close(fd);
 	}
 	return rc;
 }
@@ -875,7 +971,6 @@ int get_ipaddr(const char *ifname, struct in_addr *in)
 	return rc;
 }
 
-
 int get_ipaddrstr(const char *ifname, char *ipaddr, size_t size)
 {
 	int rc;
@@ -983,7 +1078,7 @@ int str2mac(const char *src, u8 *mac, size_t size)
 
 	memset(mac, 0, size);
 	for (i = 0; i < 6; i++, mac++)
-		if (1 != sscanf(&src[i * 3], "%02X", mac))
+		if (1 != sscanf(&src[i * 3], "%02hhX", mac))
 			goto out_err;
 	rc = 0;
 out_err:
@@ -1037,9 +1132,8 @@ int check_link_status(const char *ifname)
 		u32 data;
 	} linkstatus = { ETHTOOL_GLINK, 0};
 
-	fd = socket(PF_INET, SOCK_DGRAM, 0);
-
-	if (fd<=0)
+	fd = get_ioctl_socket();
+	if (fd < 0)
 		return retval;
 
 	memset(&ifr, 0, sizeof(ifr));
@@ -1048,7 +1142,116 @@ int check_link_status(const char *ifname)
 	if (ioctl(fd,SIOCETHTOOL, &ifr) == 0)
 		retval = linkstatus.data;
 
-	close(fd);
-
 	return retval;
 }
+
+#define NUM_ARGS 8
+
+int get_arg_val_list(char *ibuf, int ilen, int *ioff,
+			    char **args, char **argvals)
+{
+	u8 arglen;
+	u16 argvalue_len;
+	int *arglens = NULL;
+	int *argvallens = NULL;
+	int *p;
+	int numargs;
+	int i;
+
+	/* parse out args and argvals */
+	for (i = 0; ilen - *ioff > 2 * (int)sizeof(arglen); i++) {
+		if (!(i % NUM_ARGS)) {
+			p = (int *) realloc(arglens,
+				(i/NUM_ARGS + 1) * NUM_ARGS * sizeof(int));
+			if (!p) {
+				free(arglens);
+				return 0;
+			} else {
+				arglens = p;
+			}
+			p = (int *) realloc(argvallens,
+				(i/NUM_ARGS + 1) * NUM_ARGS * sizeof(int));
+			if (!p) {
+				free(argvallens);
+				return 0;
+			} else {
+				argvallens = p;
+			}
+		}
+		hexstr2bin(ibuf+*ioff, &arglen, sizeof(arglen));
+		*ioff += 2 * (int)sizeof(arglen);
+		if (ilen - *ioff >= arglen) {
+			args[i] = ibuf+*ioff;
+			*ioff += arglen;
+			*(arglens+i) = arglen;
+
+			if (ilen - *ioff > 2 * (int)sizeof(argvalue_len)) {
+				hexstr2bin(ibuf+*ioff, (u8 *)&argvalue_len,
+					   sizeof(argvalue_len));
+				argvalue_len = ntohs(argvalue_len);
+				*ioff += 2*sizeof(argvalue_len);
+				if (ilen - *ioff >= argvalue_len) {
+					argvals[i] = ibuf+*ioff;
+					*ioff += argvalue_len;
+					*(argvallens+i) = argvalue_len;
+				}
+			} else {
+				free(arglens);
+				free(argvallens);
+				return 0;
+			}
+		} else {
+			free(arglens);
+			free(argvallens);
+			return 0;
+		}
+	}
+	numargs = i;
+	for (i = 0; i < numargs; i++) {
+		args[i][*(arglens+i)] = '\0';
+		argvals[i][*(argvallens+i)] = '\0';
+	}
+	free(arglens);
+	free(argvallens);
+	return numargs;
+}
+
+int get_arg_list(char *ibuf, int ilen, int *ioff, char **args)
+{
+	u8 arglen;
+	int *arglens = NULL;
+	int *p;
+	int numargs;
+	int i;
+
+	/* parse out args */
+	for (i = 0; (ilen - *ioff > 2 * (int)sizeof(arglen)); i++) {
+		if (!(i % NUM_ARGS)) {
+			p = (int *) realloc(arglens,
+				(i/NUM_ARGS + 1) * NUM_ARGS * sizeof(int));
+			if (!p) {
+				free(arglens);
+				return 0;
+			} else {
+				arglens = p;
+			}
+		}
+		hexstr2bin(ibuf+(*ioff), &arglen, sizeof(arglen));
+		*ioff += 2*sizeof(arglen);
+		if (ilen - *ioff >= arglen) {
+			args[i] = ibuf+(*ioff);
+			*ioff += arglen;
+			*(arglens+i) = arglen;
+		} else {
+			free(arglens);
+			return 0;
+		}
+	}
+	numargs = i;
+
+	for (i = 0; i < numargs; i++)
+		args[i][*(arglens+i)] = '\0';
+
+	free(arglens);
+	return numargs;
+}
